﻿####################################################################################################
#                           NAVN      :       AAD-administration.ps1                               #
#                           VERSJON   :       1.0                                                  #
#                           LAGET AV  :       Cathinka Pedersen                                    #
#                           KONTEKST  :       Bachelorprosjekt 2019 - NTNU                         #
####################################################################################################
<#------------------------------- Referanseliste - Start -------------------------------------------
1. Avshalom (2015). Creating Tables in PowerShell. Hentet fra: 
https://stackoverflow.com/questions/32388896/creating-tables-in-powershell
2. E. Wilson og C. Liebendorfer (2009). Hey, Scripting Guy! Can I Open a File Dialog Box with Windows PowerShell? 
Hentet fra: https://blogs.technet.microsoft.com/heyscriptingguy/2009/09/01/hey-scripting
-guy-can-i-open-a-file-dialog-box-with-windows-powershell/
3. Zoltan (2012). Filedialog and PowerShell. Hentet fra: https://letitknow.wordpress.com
/2012/11/04/filedialog-and-powershell/
4. Generelt sett er AzureAD-cmdlets inspirert fra: Microsoft Azure (årstall ikke oppgitt). AzureADPreview. 
Hentet fra: https://docs.microsoft.com/en-us/powershell/module/azuread/?view=azureadps-2.0-preview
5. Generelt sett er Msol-cmdlets inspirert fra: Microsoft Azure (årstall ikke oppgitt). Azure ActiveDirectory (MSOnline). Hentet fra: 
https://docs.microsoft.com/en-us/powershell/module/msonline/?view=azureadps-1.0
6. J. Davies, D. Vangel-MSFT, dstrome2 og D. Strome (2019). View account license and service details 
with Office 365 PowerShell. Hentet fra: https://docs.microsoft.com/en-us/office365/enterprise/
powershell/view-account-license-and-service-details-with-office-365-powershell

Genrell konstruksjon av kodelinjer er inspirert fra:
7. T. L. Warner (2015). Sams Teach Yourself Windows PowerShell in 24 Hours. Sams Publishing. 
8. S. Talaat, (2015). Pro PowerShell for Microsoft Azure. Apress. 
9. A. Dhally og J. Chapman, (2015). Automating Microsoft Azure with PowerShell. Packt Publishing.
--------------------------------- Referanseliste - Slutt -------------------------------------------#>
#-------------------------------- Hjelpefunksjoner - Start -----------------------------------------
function chooseAnAlternative{
    Write-Host "`nSkriv inn ditt valg her: " -ForegroundColor Green -NoNewLine
}
function tryAgain{
    Write-Host "Ugyldig input. Prøv igjen!" -ForegroundColor Red
}
function returnToMenu{
    Write-Host "`n`n`n",`
               "---------------------------------------------------------------------------------",`
               "`n`n`t`tTrykk Enter for å returnere til menyen ", $Script:meny,"!", "`n`n"`
               "---------------------------------------------------------------------------------" `
    -ForegroundColor Yellow -NoNewLine -Separator ''
    Read-Host
    If($Script:meny -eq "Hovedmeny"){
        mainMenu
    }ElseIf($Script:meny -eq "Brukeradministrasjon"){
        userAdministration
    }ElseIf($Script:meny -eq "Gruppeadministrasjon"){
        groupAdministration
    }ElseIf($Script:meny -eq "Maskinadministrasjon"){
        deviceAdministration
    }ElseIf($Script:meny -eq "Lisensadministrasjon"){
        licenseAdministration
    }  
}
function operationSucceeded{
    loggedInUser
    Write-Host "Vellykket: " -ForegroundColor Green -NoNewLine
    Write-Host $args[0], "!" -Separator ''
}
function confirmSelectedData
{
    param([string]$operasjon, [array]$beskrivelse, [array]$variabel)
        Do{
            loggedInUser
            Write-Host $operasjon, ":`n" -ForegroundColor Magenta -Separator ''
            Write-Host "-------------------------------------------------------------------------" -ForegroundColor Yellow
            [int]$antallTegn = 30
            [int]$teller = 0
            [string]$mellomrom = ""
            ForEach($item in $beskrivelse){  
                If($item.Length -lt $antallTegn){
                    $mellomrom = $item
                    Do{
                        $mellomrom += " "
                    }Until($mellomrom.Length -eq $antallTegn)
                    $beskrivelse[$teller] = $mellomrom
                }
                $teller++
            }
            For($i=0;$i -lt $beskrivelse.Count;$i++){
           
               Write-Host "`n", $beskrivelse[$i],":", $variabel[$i] -ForegroundColor Yellow
            }
            Write-Host "-------------------------------------------------------------------------" -ForegroundColor Yellow
            Write-Host "`n`t1. Stemmer`n`t2. Skriv inn på nytt`n`t3. Avbryt" -ForegroundColor Cyan
            chooseAnAlternative
            Try{
                [int]$menyvalget = Read-Host
            }Catch{tryAgain}
        }While($menyvalget -lt 1 -or $menyvalget -gt 3)
        cls
        return $menyvalget
}
function loggingChanges
{
    param([string]$navnCSVFil, [string]$rad1, [string]$rad2)
        $rad1 | Out-File -FilePath $navnCSVFil -Append
        $rad2 | Out-File -FilePath $navnCSVFil -Append
}
function fillThisVariableWithAProperValue
{
    param([string]$infoTilBruker)
        Write-Host "`n"
        Write-Host $infoTilBruker -ForegroundColor Cyan -NoNewline
        Write-Host "`t`t`t(Trykk Enter for å avbryte)" -ForegroundColor Red
        [string]$verdi = Read-Host
        return $verdi
}
function listMenu
{
    param([string]$tittel,[array]$alleMenyvalg)
            loggedInUser
            Write-Host " - -", $tittel, "- -" -ForegroundColor Magenta
            For($i=1;$i -le $alleMenyvalg.Count;$i++){
                Write-Host "`n`t", $i, ". ",$alleMenyvalg[$i-1] -ForegroundColor Cyan -Separator ''
            }
            chooseAnAlternative
            Try{
                [int]$menyvalg = Read-Host
            }Catch{tryAgain}
        return $menyvalg
}
function error
{
    param([string]$infoTilBruker)
        loggedInUser
        Write-Host "Error: " -ForegroundColor Red -NoNewLine
        Write-Host $infoTilBruker, "!`n`n`n" -Separator ''
}
function chooseARow
{
    param([string]$tittel, [string]$objekttype, [array]$rader)
        [int]$indeks = 0
        $objektId = @{}
        $visningsnavn = @{}
        $epost = @{}
        $brukernavn = @{}
        $fornavn = @{}
        $etternavn = @{}
        $beskrivelse = @{}
        $maskinId = @{}
        $skuId = @{}
        $skuPartNumber = @{}
        $accountSkuId = @{}
        ForEach($rad in $rader){
            $indeks++
            If($rad.ObjectId -ne $null){
                $objektId.Add($indeks, $rad.ObjectId)
                $visningsnavn.Add($indeks, $rad.DisplayName)
            }Else{
                $skuId.Add($indeks, $rad.SkuId)
                $skuPartNumber.Add($indeks,$rad.SkuPartNumber)
                $accountSkuId.Add($indeks,$rad.AccountSkuId)
            }
            If($objekttype -eq "Bruker"){
                $epost.Add($indeks,$rad.UserPrincipalName)
                If($rad.GivenName -ne $null){
                    $fornavn.Add($indeks,$rad.GivenName)
                    $etternavn.Add($indeks,$rad.SurName)
                }Else{
                    $fornavn.Add($indeks,$rad.FirstName)
                    $etternavn.Add($indeks,$rad.LastName)   
                }
            }ElseIf($objekttype -eq "Sikkerhetsgruppe"){
                $brukernavn.Add($indeks, $rad.MailNickname)
                $beskrivelse.Add($indeks,$rad.Description)
            }ElseIf($objekttype -eq "Maskin"){
                $maskinId.Add($indeks,$rad.DeviceId)
            }
        }
        $indeks = 0
        $tabell = @()
        ForEach($rad in $rader){
            $indeks++
            If($objekttype -eq "Bruker"){
                $kolonner = "" | Select Indeks,Brukernavn,Navn
                $kolonner.Brukernavn = $rad.UserPrincipalName
            }ElseIf($objekttype -eq "Sikkerhetsgruppe"){
                $kolonner = '' | Select Indeks,Navn,Brukernavn,Beskrivelse
                $kolonner.Beskrivelse = $rad.Description
                $kolonner.Brukernavn = $rad.MailNickName
            }ElseIf($objekttype -eq "Maskin"){
                $kolonner = '' | Select Indeks,Navn
            }ElseIf($objekttype -eq "Lisens"){
                $kolonner = '' | Select Indeks,Lisensnavn,AntallLisenserTotalt,AntallLisenserUtdelt
                $kolonner.Lisensnavn = $rad.SkuPartNumber
                $kolonner.AntallLisenserTotalt = $rad.ActiveUnits
                $kolonner.AntallLisenserUtdelt = $rad.ConsumedUnits
            }
            $kolonner.Indeks = $indeks
            If($objekttype -ne "Lisens"){
                $kolonner.Navn = $rad.DisplayName
            }
            $tabell += $kolonner

        }
        Do{
            loggedInUser
            Write-Host $tittel, "`n" -ForegroundColor Magenta
            $tabell | Format-Table -AutoSize | Out-Host
            Write-Host "(Trykk Enter for å avbryte)" -ForegroundColor Red
            chooseAnAlternative
            Try{
                [int]$valgtRad = Read-Host
            }Catch{tryAgain}
        }Until($valgtRad -ge 1 -and $valgtRad -le $indeks -or $valgtRad -eq '')
        loggedInUser
        If($valgtRad -eq ''){return $null}
        Else{
            If($objekttype -eq "Bruker"){
                return $objektId[$valgtRad], $visningsnavn[$valgtRad], $epost[$valgtRad], $fornavn[$valgtRad], $etternavn[$valgtRad]
            }
            ElseIf($objekttype -eq "Sikkerhetsgruppe"){
                return $objektId[$valgtRad], $visningsnavn[$valgtRad], $brukernavn[$valgtRad], $beskrivelse[$valgtRad]
            }
            ElseIf($objekttype -eq "Maskin"){
                return $objektId[$valgtRad], $visningsnavn[$valgtRad], $maskinId[$valgtRad]
            }
            ElseIf($objekttype -eq "Lisens"){
                return $skuId[$valgtRad], $skuPartNumber[$valgtRad], $accountSkuId[$valgtRad]
            }
        }
} #Inspirert fra: [1]
function operationAborted
{
    param([string]$infoTilBruker)
        loggedInUser
        Write-Host "`t - - Du har valgt å avbryte operasjonen", $infoTilBruker, "- -" -ForegroundColor Red
}
function allAADUsers{
    [array]$alleAADBrukere = Get-AzureADUser | Select-Object *
    return $alleAADBrukere
}
function formattingUserName{
    param([string]$midlertidigBrukernavn)
        [string]$brukernavnet = $midlertidigBrukernavn
        $brukernavnet = $brukernavnet.ToLower()
        $brukernavnet = $brukernavnet.Replace('æ','e')
        $brukernavnet = $brukernavnet.Replace('ø','o')
        $brukernavnet = $brukernavnet.Replace('å','a')
        Return $brukernavnet
}
function loggedInUser{
    cls
    Write-Host "Innlogget bruker :", $Script:innloggetBruker
    currentTime
}
function currentTime{
    $dato = Get-Date
    $mnd = " januar", " februar", " mars", " april", " mai", " juni", " juli", " august", `
           " september", " oktober", " november", " desember"
    [int]$aktuellMnd = $dato.Month
    Write-Host "Klokkeslett      : ", $dato.Hour, ":", $dato.Minute,":", $dato.Second, "`n",`
               "Dato             : ", $dato.Day,".", $mnd[$aktuellMnd-1],".",$dato.Year, "`n`n" -Separator ''
}
#-------------------------------- Hjelpefunksjoner - Slutt -----------------------------------------
#-------------------------------- Hovedfunksjoner - Start --------------------------------------------
function connectToAzure{
    [string]$script:innloggetBruker = $null
    [bool]$avbryt = $false
    [string]$tittel = "Feil innloggingsinformasjon skrevet inn"
    [array]$alleMenyvalg = "Prøv igjen",`
                           "Avbryt"
    Do{
        $tilkoblingAzure = Get-Credential -Message 'AAD-bruker med rollen "GLOBAL ADMINISTRATOR":'
        If($tilkoblingAzure.UserName -eq $null-or $tilkoblingAzure.UserName -eq $null){
            $avbryt = $true
        }Else{
            Try{
                $tilkobling = Connect-AzureAD -Credential $tilkoblingAzure -ErrorAction SilentlyContinue
            }Catch{}
            If($tilkobling -eq $null){
                Do{
                    $menyvalg = listMenu $tittel $alleMenyvalg
                }While($menyvalg -lt 1 -or $menyvalg -gt $alleMenyvalg.Count)
                If($menyvalg -eq 2){
                    $avbryt = $true
                }
            }Else{
                $script:innloggetBruker = $tilkoblingAzure.UserName
                Connect-MsolService -Credential $tilkoblingAzure | Out-Null
	            mainMenu
            }
        }
    }While($avbryt -eq $false)
    If($avbryt -eq $true){
        cls
        Write-Host " - - Programmet avsluttes - - " -ForegroundColor Yellow
        exit
    }
}
function mainMenu{
    [string]$Script:meny = "Hovedmeny"
    [string]$tittel = "Hovedmeny"
    [array]$alleMenyvalg = "Brukeradministrasjon", `
                           "Gruppeadministrasjon", `
                           "Maskinadministrasjon",`
                           "Lisensadministrasjon", `
                           "Avslutt"
    Do{
        $menyvalg = listMenu $tittel $alleMenyvalg
    }While($menyvalg -lt 1 -or $menyvalg -gt $alleMenyvalg.Count)
    If($menyvalg -eq 1){userAdministration}
    ElseIf($menyvalg -eq 2){groupAdministration}
    ElseIf($menyvalg -eq 3){deviceAdministration}
    ElseIf($menyvalg -eq 4){licenseAdministration}
    ElseIf($menyvalg -eq 5){disconnectFromAzureAndExitScript}
    returnToMenu
}
function disconnectFromAzureAndExitScript{
    loggedInUser
    Write-Host " - - Programmet avsluttets - - " -ForegroundColor Yellow
    Disconnect-AzureAD
    Exit
}
#-------------------------------- Hovedfunksjoner - Slutt --------------------------------------------
#-------------------------------- Brukeradministrasjon - Start -------------------------------------
function userAdministration{
    [string]$Script:meny = "Brukeradministrasjon"
    [string]$tittel = "Brukeradministrasjon"
    [array]$alleMenyvalg = "Opprett ny AAD-bruker",`
                           "Nullstill passord for AAD-bruker",`
                           "Slett AAD-bruker",`
                           "List ut alle AAD-brukere",`
                           "Vis utdypende informasjon om en AAD-bruker",`
                           "Gjenopprett slettet AAD-bruker",`
                           "Tilbake til hovedmeny"
    Do{
        $brukerAdminMeny = listMenu $tittel $alleMenyvalg
    }While($brukerAdminMeny -lt 1 -or $brukerAdminMeny -gt $alleMenyvalg.Count)
    If($brukerAdminMeny -eq 1){newAADUser}
    ElseIf($brukerAdminMeny -eq 2){resetPasswordAADUser}
    ElseIf($brukerAdminMeny -eq 3){deleteAADUser}
    ElseIf($brukerAdminMeny -eq 4){listAllAADUsers}
    ElseIf($brukerAdminMeny -eq 5){informationAADUser}
    ElseIf($brukerAdminMeny -eq 6){restoreDeletedAADUser}
    ElseIf($brukerAdminMeny -eq 7){mainMenu}
    returnToMenu
}
function newAADUser{
    [string]$tittel = "Opprett ny AAD-bruker"
    [array]$alleMenyvalg =  "Enkel bruker",`
                            "Fra CSV-fil (flere)", `
                            "Avbryt"
    Do{
        $menyvalg = listMenu $tittel $alleMenyvalg
    }While($menyvalg -lt 1 -or $menyvalg -gt $alleMenyvalg.Count)
    If($menyvalg -eq 1){newAADUserSingle}
    ElseIf($menyvalg -eq 2){newAADUserPlural}
    ElseIf($menyvalg -eq 3){userAdministration}
}
function createUserName{
    [string]$midlertidigBrukernavn = ""
    $fornavnet = $Script:fornavn.Replace('-','')
    $fornavnet = $Script:fornavn.Replace(' ','')
    $etternavnet = $Script:etternavn.Replace('-','')
    $etternavnet = $Script:etternavn.Replace(' ','')
    If($fornavnet.Length -eq 2){
        $midlertidigBrukernavn = $fornavnet.Substring(0,2)
        If($Script:etternavn.Length -eq 2){
            $midlertidigBrukernavn += $etternavnet.Substring(0,2)
            $midlertidigBrukernavn += "1234"
        }ElseIf($etternavnet.Length -eq 3){
            $midlertidigBrukernavn += $etternavnet.Substring(0,3)
            $midlertidigBrukernavn += "123"
        }ElseIf($etternavnet.Length -eq 4){
            $midlertidigBrukernavn += $etternavnet.Substring(0,4)
            $midlertidigBrukernavn += "12"
        }ElseIf($etternavnet.Length -eq 5){
            $midlertidigBrukernavn += $etternavnet.Substring(0,5)
            $midlertidigBrukernavn += "1"
        }ElseIf($etternavnet.Length -ge 6){
            $midlertidigBrukernavn += $etternavnet.Substring(0,6)
        }
    }ElseIf($fornavnet.Length -eq 3){
        $midlertidigBrukernavn = $fornavnet.Substring(0,3)
        If($etternavnet.Length -eq 2){
            $midlertidigBrukernavn += $etternavnet.Substring(0,2)
            $midlertidigBrukernavn += "123"
        }ElseIf($etternavnet.Length -eq 3){
            $midlertidigBrukernavn += $etternavnet.Substring(0,3)
            $midlertidigBrukernavn += "12"
        }ElseIf($etternavnet.Length -eq 4){
            $midlertidigBrukernavn += $etternavnet.Substring(0,4)
            $midlertidigBrukernavn += "1"
        }ElseIf($etternavnet.Length -ge 5){
            $midlertidigBrukernavn += $etternavnet.Substring(0,5)
        }
    }ElseIf($fornavnet.Length -ge 4){
        $midlertidigBrukernavn += $fornavnet.Substring(0,4)
        If($etternavnet.Length -eq 2){
            $midlertidigBrukernavn += $etternavnet.Substring(0,2)
            $midlertidigBrukernavn += "12"
        }ElseIf($etternavnet.Length -eq 3){
            $midlertidigBrukernavn += $etternavnet.Substring(0,3)
            $midlertidigBrukernavn += "1"
        }ElseIf($etternavnet.Length -ge 4){
            $midlertidigBrukernavn += $etternavnet.Substring(0,4)
        }
    }
    [string]$formattertBrukernavn = formattingUserName $midlertidigBrukernavn
    [bool]$eksisterer = $True
    [int]$teller = 0
    #Løkken kjører helt til man har funnet et brukernavn som ikke eksisterer
    Do{
        $teller++
        $brukerenEksisterer = $null
        Try{
            $brukerenEksisterer = Get-AzureADUser -SearchString $formattertBrukernavn
        }Catch{}
        If($brukerenEksisterer -ne $null){
            $formattertBrukernavn += $teller.ToString()
        }Else{$eksisterer = $False}
    }While($eksisterer -eq $True)
    Return $formattertBrukernavn
}
function createPassword{
    $spesialsymbol = "~","!","@","#","$","%","^","&","*","_","-","+","="
    $storeBokstaver = "A","B","C","D","E","F","G","H","I","J","K","L","M","N","O","P","Q","R","S","T","U","V","W","X","Y","Z"
    $smaaBokstaver = "a","b","c","d","e","f","g","h","i","j","k","l","m","n","o","p","q","r","s","t","u","v","w","x","y","z"
    $tall = "0","1","2","3","4","5","6","7","8","9"
    [array]$passord = @()
    For($i=0;$i -lt 2;$i++){
        $passord += $spesialsymbol[(Get-Random -Maximum ([array]$spesialsymbol).Count)]
        $passord += $storeBokstaver[(Get-Random -Maximum ([array]$storeBokstaver).Count)]
        $passord += $smaaBokstaver[(Get-Random -Maximum ([array]$smaaBokstaver).Count)]
        $passord += $tall[(Get-Random -Maximum ([array]$tall).Count)]
    }
    [string]$passord = $passord | Sort-Object {Get-Random}
    $passord = $passord.Replace(' ','')
    Return $passord
}
function navigateToCSVFile{
    [System.Reflection.Assembly]::LoadWithPartialName("System.windows.forms") | Out-Null 
    $OpenFileDialog = New-Object System.Windows.Forms.OpenFileDialog
    $OpenFileDialog.InitialDirectory = "C:\"
    $OpenFileDialog.Filter = "CSV-filer (*.csv)|*.csv"
    $resultat = $OpenFileDialog.ShowDialog()
    If($resultat -eq "OK"){
        $stiTilCSVFil = $OpenFileDialog.Filename
        $csvFil = Get-Content $stiTilCSVFil
        $csvFil | Out-File -FilePath $stiTilCSVFil
        $csvFil = Import-CSV $stiTilCSVFil -Delimiter ";"
        return $csvFil
    }Else{return $null}
} #Inspirert fra: [2],[3]
function newAADUserSingle{
    Do{
        loggedInUser
        [int]$menyvalget = 0
        Write-Host "Opprett ny bruker ved å fylle inn følgende ...:" -ForegroundColor Magenta
        [string]$infoTilBruker = "Fornavn:"
        [string]$Script:fornavn = fillThisVariableWithAProperValue $infoTilBruker
        If($Script:fornavn -eq ""){break}
        [string]$infoTilBruker = "Etternavn:"
        [string]$Script:etternavn = fillThisVariableWithAProperValue $infoTilBruker
        If($Script:etternavn -eq ""){break}
        [string]$infoTilBruker = "Avdeling:"
        [string]$Script:avdeling = fillThisVariableWithAProperValue $infoTilBruker
        If($Script:avdeling -eq ""){break}
        [string]$infoTilBruker = "Stillingsbeskrivelse:"
        [string]$Script:stillingsbeskrivelse = fillThisVariableWithAProperValue $infoTilBruker
        If($Script:stillingsbeskrivelse -eq ""){break}
        [string]$operasjon = "Følgende bruker blir opprettet:"
        $beskrivelsesliste = @()
        $beskrivelsesliste += "NAVN", "AVDELING", "STILLINGSBESKRIVELSE"
        $variabelliste = @()
        [string]$Script:fulltNavn = $Script:fornavn, $Script:etternavn
        $variabelliste += $Script:fulltNavn, $Script:avdeling, $Script:stillingsbeskrivelse
        $menyvalget = confirmSelectedData $operasjon $beskrivelsesliste $variabelliste
        cls
    }While($menyvalget -eq 2)
    If($menyvalget -eq '' -or $menyvalget -eq 3){
        operationAborted '"Opprett ny AAD-bruker"'
    }ElseIf($menyvalget -eq 1){
        createAADUser
        [string]$beskjed = "Brukeren", $Script:brukernavn, "er opprettet"
        operationSucceeded $beskjed
    }
}
function newAADUserPlural{
    cls
    $csvFil = navigateToCSVFile
    If($csvFil -ne $null){
        ForEach($rad in $csvFil){
            [string]$Script:fornavn = $rad.Fornavn
            [string]$Script:etternavn = $rad.Etternavn
            [string]$Script:avdeling = $rad.Avdeling
            [string]$Script:stillingsbeskrivelse = $rad.Stillingsbeskrivelse
            [string]$Script:fulltNavn = $Script:fornavn,$Script:etternavn
            createAADUser
        }
        [string]$beskjed = "Brukerne fra csv-filen er nå opprettet"
        operationSucceeded $beskjed
    }Else{
        operationAborted '"Opprett ny AAD-bruker"'
    }
}
function createAADUser{
    $passordprofil = New-Object -TypeName Microsoft.Open.AzureAD.Model.PasswordProfile
    [string]$passord = createPassword
    $passordprofil.Password = $passord
    [string]$Script:brukernavn = createUserName
    [array]$domenenavnet = Get-AzureADDomain | Where-Object {$_.Name -like "*.no"} | Select Name
    [string]$domenenavn = "@"
    $domenenavn += $domenenavnet.Name
    [string]$epost = $brukernavn
    $epost += $domenenavn
     New-AzureADUser -AccountEnabled $True -PasswordProfile $passordprofil -UsageLocation "NO" `
    -Department $Script:avdeling -GivenName $Script:fornavn -Surname $Script:etternavn `
    -DisplayName $Script:fulltNavn -JobTitle $Script:stillingsbeskrivelse `
    -MailNickName $Script:brukernavn -UserType "Member" `
    -UserPrincipalName $epost -PasswordPolicies DisablePasswordExpiration | Out-Null
    Get-AzureADUser -SearchString $Script:brukernavn | Select-Object ObjectId | Set-AzureADUserPassword `
    -Password (ConvertTo-SecureString $passord -AsPlainText -Force) -ForceChangePasswordNextLogin $True | Out-Null
    #Hver bruker som opprettes, loggføres i CSV-fil på C:\:
    [string]$dato,[string]$tidspunkt = (((Get-Date -DisplayHint DateTime).ToString()).Split(' ')).Replace(' ','')
    [string]$navnCSVFil = "C:\OpprettedeAAD-brukere.csv"
    [string]$rad1CSVFil = "FulltNavn;Fornavn;Etternavn;Brukernavn;Passord;Avdeling;Epost;Dato;Tidspunkt;"
    [string]$rad2CSVFil = "$Script:fulltNavn;$Script:fornavn;$Script:etternavn;$brukernavn;$passord;$Script:avdeling;$epost;$dato;$tidspunkt;"
    loggingChanges $navnCSVFil $rad1CSVFil $rad2CSVFil
    $navnSikkerhetsgruppe = $Script:avdeling
    Try{
        $sjekkeOmSikkerhetsgruppeEksisterer = Get-AzureADGroup -SearchString $navnSikkerhetsgruppe
    }Catch{}
    If($sjekkeOmSikkerhetsgruppeEksisterer -eq $null){
        [string]$brukernavnSikkerhetsgruppe = formattingUserName $navnSikkerhetsgruppe
        [string]$beskrivelseSikkerhetsgruppe = "Alle ansatte i", $Script:avdeling.ToLower()
        $beskrivelseSikkerhetsgruppe += "savdelingen"
        New-AzureADGroup -DisplayName $navnSikkerhetsgruppe -MailEnabled $False -MailNickName $brukernavnSikkerhetsgruppe `
        -SecurityEnabled $True -Description $beskrivelseSikkerhetsgruppe | Out-Null
        [string]$navnCSVFil = "C:\OpprettedeAAD-sikkerhetsgrupper.csv"
        [string]$rad1CSVFil = "NavnSikkerhetsgruppe;Brukernavn;Beskrivelse;Dato;Tidspunkt;"
        [string]$rad2CSVFil = "$navnSikkerhetsgruppe;$brukernavnSikkerhetsgruppe;$beskrivelseSikkerhetsgruppe;$dato;$tidspunkt;"
        loggingChanges $navnCSVFil $rad1CSVFil $rad2CSVFil
    }
    $sikkerhetsgrupper = Get-AzureADGroup -SearchString $navnSikkerhetsgruppe
    ForEach($sikkerhetsgruppe in $sikkerhetsgrupper){
        $objektIdSikkerhetsgruppe = $sikkerhetsgruppe.ObjectId
        break
    }
    $nyopprettetAADBruker = Get-AzureADUser -SearchString $Script:brukernavn | Select-Object *
    $brukerObjectId = $nyopprettetAADBruker.ObjectId
    Add-AzureADGroupMember -ObjectId $objektIdSikkerhetsgruppe -RefObjectId $brukerObjectId
}
function listAADUsers
{
    param([string]$handling)
        [string]$tittel = "Velg AAD-brukeren som", $handling
        [array]$alleMenyvalg =  "Velge fra liste",`
                                "Velge basert på søk",`
                                "Avbryte"
        Do{
            $valg = listMenu $tittel $alleMenyvalg
        }While($valg -lt 1 -or $valg -gt $alleMenyvalg.Count)
        #De neste fire linjene er laget inspirert på følgende kilde: 
        #Microsoft Azure (dato ikke oppgitt). Get-AzureADDirectoryRoleMember. Hentet fra: 
        #https://docs.microsoft.com/en-us/powershell/module/azuread/get-azureaddirectoryrolemember?view=azureadps-2.0
        $globaleAdministratorer = Get-AzureADDirectoryRole | Where-Object {$_.DisplayName -eq 'Company Administrator'}
        $ekskluderGlobaleAdministratorer = @()
        ForEach($globalAdministrator in $globaleAdministratorer){
            $globAdmin = Get-AzureADDirectoryRoleMember -ObjectId $globalAdministrator.ObjectId | Get-AzureADUser
            $ekskluderGlobaleAdministratorer += $globAdmin.UserPrincipalName
        }
        If($valg -eq 1 -or $valg -eq 2){
            If($valg -eq 1){[array]$alleAADBrukere = allAADUsers}
            ElseIf($valg -eq 2){[array]$alleAADBrukere = lookUpAADUser}
            If($alleAADBrukere -eq $null){return $null}
            Else{
                $tittel = "Velg AAD-brukeren som", $handling, ":"
                ForEach($globalAdmin in $ekskluderGlobaleAdministratorer){
                    $alleAADBrukere = $alleAADBrukere | Where-Object {$_.UserPrincipalName -ne $globalAdmin}
                }
                [string]$objekttype = "Bruker"
                [array]$valgtAADBruker = chooseARow $tittel $objekttype $alleAADBrukere #0=ObjectId,1=DisplayName,2=UserPrincipalName,3=GivenName,4=SurName
                If($valgtAADBruker -eq $null){return $null}
                Else{
                    [string]$fornavn = $valgtAADBruker[3]
                    [string]$etternavn = $valgtAADBruker[4]
                    [string]$brukernavn = $valgtAADBruker[2]
                    [string]$fulltNavn = $valgtAADBruker[1]
                    [string]$objektId = $valgtAADBruker[0]
                    [string]$operasjon = "Følgende bruker", $handling
                    $beskrivelsesliste = @()
                    $beskrivelsesliste += "NAVN", "BRUKERNAVN"
                    $variabelliste = @()
                    $variabelliste += $fulltNavn,$brukernavn
                    $bekreftelseFraBruker = confirmSelectedData $operasjon $beskrivelsesliste $variabelliste
                    If($bekreftelseFraBruker -eq 1){
                        return $objektId, $fornavn, $etternavn, $brukernavn
                    }ElseIf($bekreftelseFraBruker -eq 2){listAADUsers $handling}
                    ElseIf($bekreftelseFraBruker -eq 3){return $null}
                }
            }
        }ElseIf($valg -eq 3){return $null}
}
function lookUpAADUser{
    loggedInUser
    [array]$alleAADBrukere = $null
    Do{
        Write-Host " - - Søker opp AAD-bruker - - " -ForegroundColor Magenta
        [string]$infoTilBruker = "Skriv hele brukernavnet, eller første delen av det:"
        [string]$delAvBrukernavn = fillThisVariableWithAProperValue $infoTilBruker
        cls
        If($delAvBrukernavn -eq ''){break}
        Else{
            [array]$alleAADBrukere = Get-AzureADUser -SearchString $delAvBrukernavn
            If($alleAADBrukere -ne $null){return $alleAADBrukere}
            Else{
                [string]$beskjed = 'Det finnes ingen brukernavn som begynner på "', $delAvBrukernavn, '"!',`
                                   'Skriv inn på nytt'
                error $beskjed
            }
        }
    }While($alleAADBrukere -eq $null)
    If($alleAADBrukere -eq $null){return $null}
}
function resetPasswordAADUser{
    [string]$handling = "får nytt midlertidig passord"
    [array]$valgtAADBruker = listAADUsers $handling
    If($valgtAADBruker -eq $null){operationAborted '"Nullstill passord for AAD-bruker"'}
    Else{
        [string]$passord = createPassword
        Set-AzureADUserPassword -ObjectId $valgtAADBruker[0] -Password (ConvertTo-SecureString $passord `
        -AsPlainText -Force) -ForceChangePasswordNextLogin $True | Out-Null
        [string]$brukernavn = $valgtAADBruker[3]
        [string]$fornavn = $valgtAADBruker[1]
        [string]$etternavn = $valgtAADBruker[2]
        [string]$dato,[string]$tidspunkt = ((Get-Date -DisplayHint DateTime).ToString()).Split(' ')
        [string]$navnCSVFil = "C:\NyttPassordAAD-bruker.csv"
        [string]$rad1CSVFil = "Brukernavn;Fornavn;Etternavn;Engangspassord;Dato;Tidspunkt;"
        [string]$rad2CSVFil = "$brukernavn;$fornavn;$etternavn;$passord;$dato;$tidspunkt;"
        loggingChanges $navnCSVFil $rad1CSVFil $rad2CSVFil
        [string]$beskjed = "Brukeren",$brukernavn,"må endre passord ved neste innlogging"
        operationSucceeded $beskjed
    }
}
function deleteAADUser{
    [string]$handling = "skal slettes"
    [array]$valgtAADBruker = listAADUsers $handling
    If($valgtAADBruker -eq $null){operationAborted '"Slett AAD-bruker"'}
    Else{
        Remove-AzureADUser -ObjectId $valgtAADBruker[3] | Out-Null
        [string]$brukernavn = $valgtAADBruker[3]
        [string]$fornavn = $valgtAADBruker[1]
        [string]$etternavn = $valgtAADBruker[2]
        [string]$dato,[string]$tidspunkt = ((Get-Date -DisplayHint DateTime).ToString()).Split(' ')
        [string]$navnCSVFil = "C:\SlettedeAAD-brukere.csv"
        [string]$rad1CSVFil = "Brukernavn;Fornavn;Etternavn;Dato;Tidspunkt;"
        [string]$rad2CSVFil = "$brukernavn;$fornavn;$etternavn;$dato;$tidspunkt;"
        loggingChanges $navnCSVFil $rad1CSVFil $rad2CSVFil
        [string]$beskjed = "Brukeren",$valgtAADBruker[3],"er nå slettet"
        operationSucceeded $beskjed
    }
}
function informationAADUser{
    [string]$handling = "du ønsker å vise utdypende informasjon om"
    [array]$valgtAADBruker = listAADUsers $handling
    If($valgtAADBruker -eq $null){
        operationAborted '"Vis utdypende informasjon om en AAD-bruker"'
    }
    Else{
        loggedInUser
        Write-Host " - - Utdypende informasjon om AAD-brukeren", $valgtAADBruker[3], `
        "- - `n`n`n" -ForegroundColor Magenta
        Write-Host $valgtAADBruker[1], $valgtAADBruker[2], ":" -ForegroundColor Green
        Get-AzureADUser -ObjectId $valgtAADBruker[0] | Select-Object * | Out-Host
        [array]$medlemIGrupper =  Get-AzureADUserMembership -ObjectId $valgtAADBruker[0]
        If($medlemIGrupper -eq $null){
            Write-Host "Brukeren", $valgtAADBruker[3], "er ikke medlem av noen gruppe!" `
            -ForegroundColor Yellow
        }Else{
            [int]$antallGrupper = 0
            ForEach($gruppe in $medlemIGrupper){$antallGrupper++}
            If($antallGrupper -eq 1){
                [string]$beskjed = "gruppe"
            }Else{
                [string]$beskjed = "grupper"
            }
            Write-Host "`nBrukeren ", $valgtAADBruker[3], " er medlem i følgende ", $beskjed,":" `
            -ForegroundColor Green -Separator ''
            $medlemIGrupper | Select DisplayName,Description,ObjectId | Out-Host
        }
        [array]$alleAADMaskiner = Get-AzureADDevice
        $tabell = @()
        [int]$teller = 0
        [string]$beskjed = ""
        [bool]$eierAvMaskiner = $false
        ForEach($maskin in $alleAADMaskiner){
            $eierAvDisseMaskiner = Get-AzureADDevice -ObjectId $maskin.ObjectId | Get-AzureADDeviceRegisteredOwner | Where-Object {$_.ObjectId -eq $valgtAADBruker[0]}
            ForEach($eierAvDenneMaskin in $eierAvDisseMaskiner){
                $eierAvMaskiner = $true
                $teller++
                $kolonner = "" | Select Navn,ObjektId,MaskinId
                $kolonner.Navn = $maskin.DisplayName
                $kolonner.ObjektId = $maskin.ObjectId
                $kolonner.MaskinId = $maskin.DeviceId
                $tabell += $kolonner
            }
        }
        If($eierAvMaskiner -eq $false){
            Write-Host "Brukeren", $valgtAADBruker[3], "står ikke oppført som eier av noen maskiner!" -ForegroundColor Yellow
        }Else{
            If($teller -eq 1){
                $beskjed = "denne maskinen:"
            }Else{
                $beskjed = "disse maskinene:"
            }
            Write-Host "Brukeren", $valgtAADBruker[3], "står oppført som eier av",$beskjed -ForegroundColor Green
            $tabell | Sort-Object Navn | Out-Host
        }
        $tabell = @()
        [bool]$registrertBrukerAvMaskiner = $false
        ForEach($maskin in $alleAADMaskiner){
            $registrertBrukerAvDisseMaskiner = Get-AzureADDevice -ObjectId $maskin.ObjectId | Get-AzureADDeviceRegisteredUser | Where-Object {$_.ObjectId -eq $valgtAADBruker[0]}
            ForEach($registrertBrukerAvDenneMaskin in $registrertBrukerAvDisseMaskiner){
                $registrertBrukerAvMaskiner = $true
                $teller++
                $kolonner = "" | Select Navn,ObjektId,MaskinId
                $kolonner.Navn = $maskin.DisplayName
                $kolonner.ObjektId = $maskin.ObjectId
                $kolonner.MaskinId = $maskin.DeviceId
                $tabell += $kolonner
            }
        }
        If($registrertBrukerAvMaskiner -eq $false){
            Write-Host "Brukeren", $valgtAADBruker[3], "er ikke registrert som bruker av noen maskiner!" -ForegroundColor Yellow
        }Else{
            If($teller -eq 1){
                $beskjed = "er registrert som bruker av denne maskinen:"
            }Else{
                $beskjed = "er registrert som bruker av disse maskinene:"
            }
            Write-Host "Brukeren", $valgtAADBruker[3], $beskjed -ForegroundColor Green
            $tabell | Sort-Object Navn | Out-Host
        }
        [array]$alleLisenser = get-azureaduser -ObjectId $valgtAADBruker[0] | Get-AzureADUserLicenseDetail | Select-Object ServicePlans 
        If($alleLisenser -eq $null){
            Write-Host "Brukeren", $valgtAADBruker[3], "er ikke tildelt noen lisenser!" -ForegroundColor Yellow
        }Else{
            [int]$antallLisenser = 0
            [string]$beskjed = ""
            $alleLisenser | ForEach-Object {$antallLisenser++}
            If($antallLisenser -eq 1){$beskjed = "lisensplan:"}
            Else{$beskjed = "lisensplaner:"}
            Write-Host "Brukeren", $valgtAADBruker[3], "er tildelt følgende", $beskjed -ForegroundColor Green
            $alleLisenser | Format-List
            [array]$alleLisenser = (Get-AzureADUser -ObjectId $valgtAADBruker[0]).Assignedplans
            $unikeLisenser = $alleLisenser | Select-Object -Property Service,AssignedTimeStamp,ServicePlanId,CapabilityStatus | Sort-Object Service -Unique
            Write-Host "Brukeren", $valgtAADBruker[3], "har tilgang på følgende tjenester:" -ForegroundColor Green
            $unikeLisenser | Out-Host 
        }
    }
}
function listAllAADUsers{
    loggedInUser
    Write-Host "`n`t`t`t - - Lister ut alle AAD-brukere - - `n" -ForegroundColor Magenta
    [array]$alleAADBrukere = allAADUsers
    $alleAADBrukere | Select-Object DisplayName,UserPrincipalName,UserType `
    | Format-Table -Autosize | Out-Host
}
function restoreDeletedAADUser{
    [bool]$avbruttOperasjon = $false
    [string]$tittel = "Velg AAD-bruken du ønsker å gjenopprette"
    [string]$objekttype = "Bruker"
    [int]$bekreftelseFraBruker = 0
    [array]$gjenopprettMeg = $null
    [array]$slettedeAADBrukere = Get-MsolUser -All -ReturnDeletedUsers | Select-Object ObjectId,DisplayName,UserPrincipalName,FirstName,LastName
    Do{
        [array]$gjenopprettMeg = chooseARow $tittel $objekttype $slettedeAADBrukere #0=ObjectId,1=DisplayName,2=UserPrincipalName,3=FirstName,4=LastName
        If($gjenopprettMeg -eq $null){
            $avbruttOperasjon = $true
            break
        }Else{
            [string]$fornavn = $gjenopprettMeg[3]
            [string]$etternavn = $gjenopprettMeg[4]
            [string]$brukernavn = $gjenopprettMeg[2]
            [string]$fulltNavn = $gjenopprettMeg[1]
            [string]$operasjon = "Følgende bruker blir gjenopprettet:"
            $beskrivelsesliste = @()
            $beskrivelsesliste += "NAVN", "BRUKERNAVN"
            $variabelliste = @()
            $variabelliste += $fulltNavn,$brukernavn
            $bekreftelseFraBruker = confirmSelectedData $operasjon $beskrivelsesliste $variabelliste
            If($bekreftelseFraBruker -eq 3){
                $avbruttOperasjon = $true
                break
            }

        }
    }While($bekreftelseFraBruker -eq 2)
    If($avbruttOperasjon -eq $true){
        operationAborted '"Gjenopprett slettet AAD-bruker"'
    }Else{
        [string]$dato,[string]$tidspunkt = ((Get-Date -DisplayHint DateTime).ToString()).Split(' ')
        [string]$navnCSVFil = "C:\GjenopprettedeAAD-brukere.csv"
        [string]$rad1CSVFil = "Brukernavn;Fornavn;Etternavn;Dato;Tidspunkt;"
        [string]$rad2CSVFil = "$brukernavn;$fornavn;$etternavn;$dato;$tidspunkt;"
        loggingChanges $navnCSVFil $rad1CSVFil $rad2CSVFil
        Restore-MsolUser -UserPrincipalName $gjenopprettMeg[2] | Out-Null
        [string]$beskjed = "Brukeren", $gjenopprettMeg[2], "er nå gjenopprettet"
        operationSucceeded $beskjed
    }
}
#-------------------------------- Brukeradministrasjon - Slutt -------------------------------------
#-------------------------------- Gruppeadministrasjon - Start -------------------------------------
function groupAdministration{
    [array]$sjekkeOmFinnesSikkerhetsgrupper = $null
    Try{
        $sjekkeOmFinnesSikkerhetsgrupper = Get-AzureADGroup
    }Catch{}
    If($sjekkeOmFinnesSikkerhetsgrupper -eq $null){
        [string]$feilmelding = "Det er ingen sikkerhetsgrupper i AAD!`n",`
                                "`t   Denne funksjonaliteten krever at det finnes sikkerhetsgrupper i AAD.`n",`
                                "`t   Legg til noen AAD-sikkerhetsgrupper, og prøv igjen"
        error $feilmelding
    }Else{
        [string]$Script:meny = "Gruppeadministrasjon"
        [string]$tittel = "Gruppeadministrasjon"
        [array]$alleMenyvalg =  "Opprett ny AAD-sikkerhetsgruppe",`
                                "Rediger AAD-sikkerhetsgruppe",`
                                "List ut medlemmer i AAD-sikkerhetsgruppe",`
                                "Slett AAD-sikkerhetsgruppe",`
                                "List ut alle AAD-sikkerhetsgrupper",`
                                "Vis utdypende informasjon om AAD-sikkerhetsgruppe", `
                                "Tilbake til hovedmeny"
        Do{
            $menyGruppeAdmin = listMenu $tittel $alleMenyvalg
        }While($menyGruppeAdmin -lt 1 -or $menyGruppeAdmin -gt $alleMenyvalg.Count)

            If($menyGruppeAdmin -eq 1){newAADGroup}
            ElseIf($menyGruppeAdmin -eq 2){changeAADGroup}
            ElseIf($menyGruppeAdmin -eq 3){listAADGroupMembers}
            ElseIf($menyGruppeAdmin -eq 4){deleteAADGroup}
            ElseIf($menyGruppeAdmin -eq 5){listAllAADGroups}
            ElseIf($menyGruppeAdmin -eq 6){informationAADGroup}
            ElseIf($menyGruppeAdmin -eq 7){mainMenu}
    }
    returnToMenu
}
function newAADGroup{
    loggedInUser
    [array]$eksisterendeGrupper = Get-AzureADGroup | Select-Object DisplayName
    [array]$alleGruppenavn = $eksisterendeGrupper.DisplayName
    [bool]$gruppenavnEksisterer = $null
    [string]$avbruttOperasjon = '"Opprett ny AAD-sikkerhetsgruppe"'
    Write-Host " - - Opprett ny sikkerhetsgruppe ved å fylle inn følgende: - -" -ForegroundColor Magenta
    Do{ 
        [string]$infoTilBruker = "Navn:"
        $gruppenavn = fillThisVariableWithAProperValue $infoTilBruker
        If($alleGruppenavn -contains $gruppenavn){
            loggedInUser
            $gruppenavnEksisterer = $True
            Write-Host "Opprett ny gruppe ved å fylle inn følgende:" -ForegroundColor Magenta
            [string]$beskjed = "Gruppenavnet",$gruppenavn,"er allerede i bruk!`nForsøk igjen"
            error $beskjed
        }Else{$gruppenavnEksisterer = $False}
    }While($gruppenavnEksisterer -eq $True)
    If($gruppenavn -eq ''){operationAborted $avbruttOperasjon}
    Else{
        [string]$infoTilBruker = "Beskrivelse"
        $beskrivelse = fillThisVariableWithAProperValue $infoTilBruker
        If($beskrivelse -eq ''){operationAborted $avbruttOperasjon}
        Else{
            [string]$operasjon = "Følgende sikkerhetsgruppe blir opprettet"
            $variabelbeskrivelse = @()
            $variabelbeskrivelse += "NAVN", "BESKRIVELSE"
            $variabelnavn = @()
            $variabelnavn += $gruppenavn, $beskrivelse
            $menyvalget = confirmSelectedData $operasjon $variabelbeskrivelse $variabelnavn
            loggedInUser
            If($menyvalget -eq 1){
                [string]$brukernavn = formattingUserName $gruppenavn
                If($brukernavn -contains ' '){
                    $brukernavn = $brukernavn.Replace(' ','')
                }
                New-AzureADGroup -DisplayName $gruppenavn -MailEnabled $False -MailNickName $brukernavn `
                -SecurityEnabled $True -Description $beskrivelse | Out-Null
                [string]$beskjed = "Gruppen", $gruppenavn,"er opprettet"
                operationSucceeded $beskjed
                #Oppretting av ny gruppe loggføres i  CSV-fil på C:\:
                [string]$dato,[string]$tidspunkt = ((Get-Date -DisplayHint DateTime).ToString()).Split(' ')
                [string]$navnCSVFil = "C:\OpprettedeAAD-sikkerhetsgrupper.csv"
                [string]$rad1CSVFil = "Navn;Brukernavn;Beskrivelse;Dato;Tidspunkt;"
                [string]$rad2CSVFil = "$gruppenavn;$brukernavn;$beskrivelse;$dato;$tidspunkt;"
                loggingChanges $navnCSVFil $rad1CSVFil $rad2CSVFil
            }ElseIf($menyvalget -eq 2){newAADGroup}
            ElseIf($menyvalget -eq 3){operationAborted $avbruttOperasjon}
        }
    }
}
function changeAADGroup{
    [string]$tittel = "Endre AAD-sikkerhetsgruppe"
    [array]$alleMenyvalg =  "Endre beskrivelse på gruppe",`
                            "Legg til en bruker i gruppe", `
                            "Fjern et medlem fra gruppe",`
                            "Overfør ett medlem fra en gruppe til en annen",`
                            "Overfør alle medlemmer i en gruppe til en annen",`
                            "Avbryt"
    Do{
        $menyEndreAADGruppe = listMenu $tittel $alleMenyvalg
    }While($menyEndreAADGruppe -lt 1 -or $menyEndreAADGruppe -gt $alleMenyvalg.Count)
    If($menyEndreAADGruppe -eq 6){groupAdministration}
    Else{
        [string]$handling = ""
        If($menyEndreAADGruppe -eq 1){$handling = "det skal endres beskrivelse på"}
        ElseIf($menyEndreAADGruppe -eq 2){$handling = "det skal legges til en bruker i"}
        ElseIf($menyEndreAADGruppe -eq 3){$handling = "det skal fjernes en bruker fra"}
        ElseIf($menyEndreAADGruppe -eq 4){$handling = "ett av medlemmene skal overføres til en ny sikkerhetsgruppe fra"}
        ElseIf($menyEndreAADGruppe -eq 5){$handling = "alle medlemmene skal overføres til en ny sikkerhetsgruppe fra"}
        [array]$valgtAADGruppe = listAADGroups $handling #0=ObjectId, 1=MailNickName, 2=Description, 3=DisplayName
        If($valgtAADGruppe -eq $null){operationAborted '"Endre AAD-sikkerhetsgruppe"'}
        Else{
            If($menyEndreAADGruppe -eq 1){
                changeDescriptionAADGroup $valgtAADGruppe
            }ElseIf($menyEndreAADGruppe -eq 2){
                $handling = "skal legges til i sikkerhetsgruppen", $valgtAADGruppe[3]
                [array]$leggTilMeg = listAADUsers $handling #0=objekt-id,1=fornavn,2=etternavn,3=brukernavn
                If($leggTilMeg -ne $null){
                    populateAADGroups $leggTilMeg[0] $leggTilMeg[3] $valgtAADGruppe[0] $valgtAADGruppe[3]
                }Else{
                    [string]$avbruttOperasjon = 'Du har valgt å avbryte operasjonen "Legg til bruker i sikkerhetsgruppe"'
                    operationAborted $avbruttOperasjon
                }
            }ElseIf($menyEndreAADGruppe -eq 3){
                removeAMemberFromThisAADGroup $valgtAADGruppe
            }ElseIf($menyEndreAADGruppe -eq 4){
                transferOneMemberToAnOtherAADGroup $valgtAADGruppe
            }ElseIf($menyEndreAADGruppe -eq 5){
                transferAllMembersToAnOtherAADGroup $valgtAADGruppe
            }
        }
    }
}
function changeDescriptionAADGroup
{
    param([array]$valgtSikkerhetsgruppe)
        loggedInUser
        Write-Host " - - Ny beskrivelse for sikkerhetsgruppen", $valgtSikkerhetsgruppe[3], ":" -ForegroundColor Magenta
        [string]$infoTilBruker = "Beskrivelse: "
        $nyBeskrivelse = fillThisVariableWithAProperValue $infoTilBruker
        If($nyBeskrivelse -eq ''){operationAborted '"Endre beskrivelse AAD-sikkerhetsgruppe"'}
        Else{
            [string]$dato,[string]$tidspunkt = ((Get-Date -DisplayHint DateTime).ToString()).Split(' ')
            [string]$forrigeBeskrivelse = $valgtSikkerhetsgruppe[2]
            [string]$navnSikkerhetsgruppe = $valgtSikkerhetsgruppe[3]
            [string]$brukernavnSikkerhetsgruppe = $valgtSikkerhetsgruppe[1] 
            [string]$navnCSVFil = "C:\NyBeskrivelseAAD-sikkerhetsgruppe.csv"
            [string]$rad1CSVFil = "AAD-sikkerhetsgruppe;ForrigeBeskrivelse;NyBeskrivelse;Dato;Tidspunkt;"
            [string]$rad2CSVFil = "$navnSikkerhetsgruppe;$forrigeBeskrivelse;$nyBeskrivelse;$dato;$tidspunkt;"
            loggingChanges $navnCSVFil $rad1CSVFil $rad2CSVFil
            Set-AzureADGroup -ObjectId $valgtSikkerhetsgruppe[0] -Description $nyBeskrivelse | Out-Null
            [string]$beskjed = "Sikkerhetsgruppen", $valgtSikkerhetsgruppe[3], "har nå fått ny beskrivelse"
            operationSucceeded $beskjed
        }
}
function removeAMemberFromThisAADGroup
{
    param([array]$valgtSikkerhetsgruppe)
        [string]$avbruttOperasjon = '"Fjern et medlem fra AAD-sikkerhetsgruppe"'
        Do{
            loggedInUser
            $gruppemedlemmer = Get-AzureADGroupMember -ObjectId $valgtSikkerhetsgruppe[0]
            [string]$tittel = "Velg den brukeren du ønsker å fjerne fra sikkerhetsgruppen", $valgtSikkerhetsgruppe[3], ":`n"
            [string]$objekttype = "Bruker"
            [array]$fjernBruker = chooseARow $tittel $objekttype $gruppemedlemmer #0=ObjectId,1=DisplayName,2=UserPrincipalName,3=GivenName,4=SurName
            If($fjernBruker -eq $null){break}
            Else{
                [string]$fornavn = $fjernBruker[3]
                [string]$etternavn = $fjernBruker[4]
                [string]$brukernavn = $fjernBruker[2]
                [string]$fulltNavn = $fjernBruker[1]
                [string]$objektIdBruker = $fjernBruker[0]
                [string]$operasjon = "Følgende bruker blir slettet fra sikkerhetsgruppen", $valgtAADGruppe[3]
                $variabelbeskrivelse = @()
                $variabelbeskrivelse += "NAVN", "BRUKERNAVN"
                $variabelnavn = @()
                $variabelnavn += $fulltNavn,$brukernavn
                $brukervalg = confirmSelectedData $operasjon $variabelbeskrivelse $variabelnavn
                If($brukervalg -eq 1 -or $brukervalg -eq 3){break}
            }
        }While($brukervalg -eq 2)
        If($brukervalg -eq 1){
            Remove-AzureADGroupMember -ObjectId $valgtSikkerhetsgruppe[0] -MemberId $objektIdBruker `
            | Out-Null
            [string]$infoTilBruker = "Brukeren", $brukernavn, "er nå slettet fra sikkerhetsgruppen", $valgtSikkerhetsgruppe[3]
            operationSucceeded $infoTilBruker
        }Else{operationAborted $avbruttOperasjon}

}
function transferOneMemberToAnOtherAADGroup
{
    param([array]$valgtSikkerhetsgruppe)
        [string]$avbruttOperasjon = '"Overfør et medlem fra en gruppe til en annen"'
        [string]$feilmelding = '"Det er ingen medlemmer i sikkerhetsgruppen"', $valgtSikkerhetsgruppe[3]
        $medlemmer = Get-AzureADGroupMember -ObjectId $valgtSikkerhetsgruppe[0] | Select-Object *
        If($medlemmer -eq $null){error $feilmelding}
        Else{
            [string]$tittel = " - - Velg den brukeren du ønsker å overføre til en annen sikkerhetsgruppe - - `n`n",`
                                "`t`tFølgende brukere er medlemmer i sikkerhetsgruppen", $valgtSikkerhetsgruppe[3], ":"
            [string]$objekttype = "Bruker"
            [int]$brukervalg = 0
            Do{
                [array]$fjernBruker = chooseARow $tittel $objekttype $medlemmer #0=ObjectId,1=DisplayName,2=UserPrincipalName,3=GivenName,4=SurName
                If($fjernBruker -eq $null){break}
                Else{
                    [string]$fornavn = $fjernBruker[3]
                    [string]$etternavn = $fjernBruker[4]
                    [string]$brukernavn = $fjernBruker[2]
                    [string]$fulltNavn = $fjernBruker[1]
                    [string]$objektIdBruker = $fjernBruker[0]
                    [string]$operasjon = "Følgende bruker blir slettet fra sikkerhetsgruppen", $valgtAADGruppe[3], "og lagt til i en ny sikkerhetsgruppe"
                    $variabelbeskrivelse = @()
                    $variabelbeskrivelse += "NAVN", "BRUKERNAVN"
                    $variabelnavn = @()
                    $variabelnavn += $fulltNavn,$brukernavn
                    $brukervalg = confirmSelectedData $operasjon $variabelbeskrivelse $variabelnavn
                }
            }While($brukervalg -eq 2)
            If($brukervalg -eq 3 -or $brukervalg -eq 0){operationAborted $avbruttOperasjon}
            Else{
                [string]$handling = $brukernavn, "skal overføres til"
                [array]$tilAADGruppe = listAADGroups $handling #0=ObjectId, 1=MailNickName, 2=Description, 3=DisplayName
                If($tilAADGruppe -eq $null){operationAborted $avbruttOperasjon}
                Else{
                    [string]$fragruppensObjektId = $valgtSikkerhetsgruppe[0]
                    [string]$tilgruppensObjektId = $tilAADGruppe[0]
                    Remove-AzureADGroupMember -ObjectId $fragruppensObjektId -MemberId $objektIdBruker | Out-Null
                    $sjekkeOmFinnes = Get-AzureADGroup -ObjectId $tilgruppensObjektId | Get-AzureADGroupMember | Where {$_.ObjectId -eq $objektIdBruker}
                    If($sjekkeOmFinnes -eq $null){
                        Add-AzureADGroupMember -ObjectId $tilgruppensObjektId -RefObjectId $objektIdBruker | Out-Null
                    }
                    [string]$beskjed = "Brukeren", $brukernavn, "er nå overført fra sikkerhetsgruppen", $valgtSikkerhetsgruppe[3], "til sikkerhetsgruppen", $tilAADGruppe[3]
                    operationSucceeded $beskjed
                }
            }
        }
}
function transferAllMembersToAnOtherAADGroup
{
    param([array]$valgtSikkerhetsgruppe)
        [string]$avbruttOperasjon = '"Overfør alle medlemmer i en sikkerhetsgruppe til en annen sikkerhetsgruppe"'
        [string]$feilmelding = '"Det er ingen medlemmer i sikkerhetsgruppen"', $valgtSikkerhetsgruppe[3]
        [array]$medlemmerFraAADGruppe = Get-AzureADGroupMember -ObjectId $valgtSikkerhetsgruppe[0] | Select-Object *
        [array]$muligeTilGrupper = Get-AzureADGroup | Where-Object {$_.ObjectId -ne $valgtSikkerhetsgruppe[0]}
        If($medlemmerFraAADGruppe -eq $null){error $feilmelding}
        Else{
            [string]$tittel = " - - Alle medlemmene i sikkerhetsgruppen",$valgtSikkerhetsgruppe[3], "skal overføres til følgende sikkerhetsgruppe - - "
            [string]$objekttype = "Sikkerhetsgruppe"
            Do{
                [array]$tilAADGruppe = chooseARow $tittel $objekttype $muligeTilGrupper #0=ObjectId,1=DisplayName,2=MailNickName,3=Description
                If($tilAADGruppe -eq $null){break}
                Else{
                    [string]$brukernavn = $tilAADGruppe[2]
                    [string]$fulltNavn = $tilAADGruppe[1]
                    [string]$beskrivelse = $tilAADGruppe[3]
                    [string]$operasjon = "Alle medlemmene i sikkerhetsgruppen", $valgtAADGruppe[3], "blir lagt til i sikkerhetsgruppen:"
                    $variabelbeskrivelse = @()
                    $variabelbeskrivelse += "NAVN", "BRUKERNAVN", "BESKRIVELSE"
                    $variabelnavn = @()
                    $variabelnavn += $fulltNavn,$brukernavn,$beskrivelse
                    [int]$brukervalg = confirmSelectedData $operasjon $variabelbeskrivelse $variabelnavn
                }
            }While($brukervalg -eq 2)
            If($tilAADGruppe -eq $null -or $brukervalg -eq 3){operationAborted $avbruttOperasjon}
            Else{
                [string]$fragruppensObjektId = $valgtSikkerhetsgruppe[0]
                [string]$tilgruppensObjektId = $tilAADGruppe[0]
                [array]$medlemmerITilAADGruppe = Get-AzureADGroupMember -ObjectId $tilgruppensObjektId | Select-Object *
                ForEach($medlem in $medlemmerFraAADGruppe){
                    Remove-AzureADGroupMember -ObjectId $fragruppensObjektId -MemberId $medlem.ObjectId | Out-Null
                }
                ForEach($medlem in $medlemmerITilAADGruppe){
                    $medlemmerFraAADGruppe = $medlemmerFraAADGruppe | Where-Object {$_.ObjectId -ne $medlem.ObjectId}
                }
                ForEach($medlem in $medlemmerFraAADGruppe){
                    Add-AzureADGroupMember -ObjectId $tilgruppensObjektId -RefObjectId $medlem.ObjectId | Out-Null
                }
                [string]$beskjed = "Alle medlemmene i sikkerhetsgruppen", $valgtSikkerhetsgruppe[3], "til sikkerhetsgruppen", $tilAADGruppe[1]
                operationSucceeded $beskjed
                }
            }
}
function listAADGroupMembers{
    [string]$handling = "du ønsker å se alle medlemmer i"
    [array]$valgtAADGruppe = listAADGroups $handling
    [string]$avbruttOperasjon = '"List ut alle medlemmer i AAD-sikkerhetsgruppe"'
    If($valgtAADGruppe -eq $null){operationAborted $avbruttOperasjon}
    Else{
        [int]$teller = 0
        [array]$alleMedlemmer = Get-AzureADGroupMember -ObjectId $valgtAADGruppe[0]
        $alleMedlemmer | Select ObjectId | ForEach-Object {$teller++}
        loggedInUser
        Write-Host "Det er totalt", $teller, " medlemmer i sikkerhetsgruppen ", $valgtAADGruppe[3], ":" -NoNewline -ForegroundColor Magenta
        Write-Host "`n`n`n"
        If($alleMedlemmer -ne $null){
            [array]$medlemmerBrukere = $alleMedlemmer | Where-Object {$_.ObjectType -eq 'User'} | Select -Property ObjectType,DisplayName,UserPrincipalName,ObjectId
            [array]$medlemmerGrupper = $alleMedlemmer | Where-Object {$_.ObjectType -eq 'Group'} | Select -Property ObjectType,DisplayName,MailNickName,Description,ObjectId
            [array]$medlemmerMaskiner = $alleMedlemmer | Where-Object {$_.ObjectType -eq 'Device'} | Select -Property DisplayName,DeviceOSType,DeviceOSVersion,DeviceId,ObjectId
            listMembersMemberTypeUser $valgtAADGruppe $medlemmerBrukere
            listMembersMemberTypeGroup $valgtAADGruppe $medlemmerGrupper
            listMembersMemberTypeDevice $valgtAADGruppe $medlemmerMaskiner
        }
    }
}
function deleteAADGroup{
    [string]$handling = "skal slettes"
    [array]$valgtAADGruppe = listAADGroups $handling #0=ObjectId, 1=MailNickName, 2=Description, 3=DisplayName
    [string]$avbruttOperasjon = '"Slett AAD-sikkerhetsgruppe"'
    If($valgtAADGruppe -eq $null){operationAborted $avbruttOperasjon}
    Else{
        [string]$gruppenavn = $valgtAADGruppe[3]
        [string]$brukernavn = $valgtAADGruppe[1]
        [string]$beskrivelse = $valgtAADGruppe[2]
        Remove-AzureADGroup -ObjectId $valgtAADGruppe[0] | Out-Null
        [string]$dato,[string]$tidspunkt = ((Get-Date -DisplayHint DateTime).ToString()).Split(' ')
        [string]$navnCSVFil = "C:\SlettedeAAD-sikkerhetsgrupper.csv"
        [string]$rad1CSVFil = "Navn;Brukernavn;Beskrivelse;Dato;Tidspunkt;"
        [string]$rad2CSVFil = "$gruppenavn;$brukernavn;$beskrivelse;$dato;$tidspunkt;"
        loggingChanges $navnCSVFil $rad1CSVFil $rad2CSVFil
        [string]$operasjon = "Sikkerhetsgruppen", $valgtAADGruppe[3], "er nå slettet"
        operationSucceeded $operasjon
    }
}
function listAADGroups{
    param([string]$handling)
        [string]$tittel = "Velg AAD-gruppen som", $handling
        [array]$alleMenyvalg =  "Velge gruppe fra liste",`
                                "Velge gruppe basert på søk",`
                                "Avbryte"
        Do{
            $valg = listMenu $tittel $alleMenyvalg
        }While($valg -lt 1 -or $valg -gt $alleMenyvalg.Count)
        If($valg -eq 3){return $null}
        Else{
            If($valg -eq 1){[array]$alleAADGrupper = Get-AzureADGroup | Select-Object ObjectId,DisplayName,Description,MailNickName}
            ElseIf($valg -eq 2){
                loggedInUser
                Do{
                    Write-Host "Velg AAD-gruppen som", $handling, ":" -ForegroundColor Magenta
                    [string]$infoTilBruker = "Skriv hele gruppenavnet eller førsten av gruppenavnet her:"
                    [string]$delAvGruppenavn = fillThisVariableWithAProperValue $infoTilBruker
                    If($delAvGruppenavn -eq ''){return $null}
                    Else{
                        [array]$alleAADGrupper = $null
                        Try{
                            $alleAADGrupper = Get-AzureADGroup -SearchString $delAvGruppenavn
                        }Catch{}
                        If($alleAADGrupper -eq $null){
                            [string]$feilmelding = 'Det finnes ingen AAD-sikkerhetsgruppe som starter på "', $delAvGruppenavn, '"!','Forsøk igjen'
                            error $feilmelding
                        }
                    }
                }While($alleAADGrupper -eq $null)
            }
            [string]$tittel = "Velg AAD-gruppen som", $handling, ":" 
            [string]$objekttype = "Sikkerhetsgruppe"
            [array]$valgtAADGruppe = chooseARow $tittel $objekttype $alleAADGrupper #0=ObjectId,1=DisplayName,2=MailNickName,3=Description
            If($valgtAADGruppe -eq $null){return $null}
            Else{
                [string]$brukernavn = $valgtAADGruppe[2]
                [string]$beskrivelse = $valgtAADGruppe[3]
                [string]$gruppenavn = $valgtAADGruppe[1]
                [string]$objektId = $valgtAADGruppe[0]
                [string]$operasjon = "Du har valgt følgende gruppe som", $handling
                $variabelbeskrivelse = @()
                $variabelbeskrivelse += "NAVN", "BRUKERNAVN","BESKRIVELSE"
                $variabelnavn = @()
                $variabelnavn += $gruppenavn, $brukernavn, $beskrivelse
                $bekreftelseFraBruker = confirmSelectedData $operasjon $variabelbeskrivelse $variabelnavn
                If($bekreftelseFraBruker -eq 1){
                    [array]$gruppeinformasjon = $objektId, $brukernavn, $beskrivelse, $gruppenavn
                    return $gruppeinformasjon
                }ElseIf($bekreftelseFraBruker -eq 2){listAADGroups $handling}
                ElseIf($bekreftelseFraBruker -eq 3){return $null}
            }
        }
}
function populateAADGroups{
    $alleredeMedlemmer = Get-AzureADGroupMember -ObjectId $args[2]
    If($alleredeMedlemmer.ObjectId -contains $args[0]){
        [string]$beskjed = $args[1], "er allerede medlem i sikkerhetsgruppen", $args[3]
        error $beskjed
    }Else{
        Add-AzureADGroupMember -ObjectId $args[2] -RefObjectId $args[0]
        [string]$beskjed = "Brukeren", $args[1], "er lagt til i", $args[3]
        operationSucceeded $beskjed
    }
}
function informationAADGroup{
    [string]$handling = "du ønsker å se utdypende informasjon om"
    [array]$valgtAADGruppe = listAADGroups $handling #0=ObjectId, 1=MailNickName, 2=Description, 3=DisplayName
    [string]$avbruttOperasjon = '"Vis utdypende informasjon om AAD-sikkerhetsgruppe"'
    If($valgtAADGruppe -eq $null){operationAborted $avbruttOperasjon}
    Else{
        loggedInUser
        Write-Host " - - Utdypende informasjon om AAD-gruppen", $valgtAADGruppe[3], `
        "- - `n`n`n" -ForegroundColor Magenta
        Write-Host $valgtAADGruppe[3],":" -ForegroundColor Green
        Get-AzureADGroup -ObjectId $valgtAADGruppe[0] | Select-Object * | Out-Host
        [array]$lisens = (Get-MsolGroup -ObjectId $valgtAADGruppe[0]).licenses | select skupartnumber
        If($lisens -eq $null){
            Write-Host " - -Sikkerhetsgruppen", $valgtAADGruppe[3],"er ikke tildelt noen lisens: - -`n`n" -ForegroundColor Yellow
        }Else{
           Write-Host "Sikkerhetsgruppen", $valgtAADGruppe[3], "er tildelt følgende lisens(er):`n" -ForegroundColor Green
           $lisens | Out-Host
        }
        [array]$medlemAvGrupper = Get-AzureADGroup | Get-AzureADGroupMember | Where-Object {$_.ObjectId -eq $valgtAADGruppe[0]}
        If($medlemAvGrupper -eq $null){
            Write-Host " - - Sikkerhetsgruppen", $valgtAADGruppe[3], "er ikke medlem av noen sikkerhetsgruppe - -`n`n" -ForegroundColor Yellow
        }Else{
            Write-Host "Sikkerhetsgruppen", $valgtAADGruppe[3], "er medlem av følgende sikkerhetsgrupper:" -ForegroundColor Green
            $medlemAvGrupper | Out-Host
        }
        [array]$gruppemedlemmer = Get-AzureADGroupMember -ObjectId $valgtAADGruppe[0] | Select *
        [array]$medlemmerBrukere = $gruppemedlemmer | Where-Object {$_.ObjectType -eq 'User'} | Select -Property ObjectType,DisplayName,UserPrincipalName,ObjectId
        [array]$medlemmerGrupper = $gruppemedlemmer | Where-Object {$_.ObjectType -eq 'Group'} | Select -Property ObjectType,DisplayName,MailNickName,Description,ObjectId
        [array]$medlemmerMaskiner = $gruppemedlemmer | Where-Object {$_.ObjectType -eq 'Device'}| Select -Property DisplayName,DeviceOSType,DeviceOSVersion,DeviceId,ObjectId
        If($gruppemedlemmer -eq $null){
            Write-Host "- - Sikkerhetsgruppen", $valgtAADGruppe[3], "har ingen medlemmer - -`n`n" `
            -ForegroundColor Yellow
        }Else{
            listMembersMemberTypeUser $valgtAADGruppe $medlemmerBrukere
            listMembersMemberTypeGroup $valgtAADGruppe $medlemmerGrupper
            listMembersMemberTypeDevice $valgtAADGruppe $medlemmerMaskiner
        }
    }
}
function listMembersMemberTypeUser
{
    param([array]$valgtSikkerhetsgruppe, [array]$gruppemedlemmer)
        If($gruppemedlemmer -eq $null){
            Write-Host "Det er ingen AAD-brukere i sikkerhetsgruppen", $valgtSikkerhetsgruppe[3], ":`n`n" -ForegroundColor Yellow
        }Else{
            Write-Host "Det er", $gruppemedlemmer.Count, "AAD-brukere i sikkerhetsgruppen", $valgtSikkerhetsgruppe[3], ":`n" -ForegroundColor Green
            [int]$indeks = 0
            $tabell = @()
            ForEach($gruppemedlem in $gruppemedlemmer){
                $indeks++
                $rader = "" | Select Indeks,Navn,Epostadresse,ObjektId
                $rader.Indeks = $indeks
                $rader.Navn = $gruppemedlem.DisplayName
                $rader.Epostadresse = $gruppemedlem.UserPrincipalName
                $rader.ObjektId = $gruppemedlem.ObjectId
                $tabell += $rader
            }
            $tabell | Format-Table -AutoSize | Out-Host
        }
}
function listMembersMemberTypeGroup
{
    param([array]$valgtSikkerhetsgruppe, [array]$gruppemedlemmer)
        If($gruppemedlemmer -eq $null){
            Write-Host "Det er ingen AAD-grupper i sikkerhetsgruppen", $valgtSikkerhetsgruppe[3], ":`n`n" -ForegroundColor Yellow
        }Else{
            Write-Host "Det er", $gruppemedlemmer.Count, "AAD-grupper i sikkerhetsgruppen", $valgtSikkerhetsgruppe[3], ":`n" -ForegroundColor Green
            [int]$indeks = 0
            $tabell = @()
            ForEach($gruppemedlem in $gruppemedlemmer){
                $indeks++
                $rader = "" | Select Indeks,Navn,Brukernavn,Beskrivelse,ObjektId
                $rader.Indeks = $indeks
                $rader.Navn = $gruppemedlem.DisplayName
                $rader.Brukernavn = $gruppemedlem.UserPrincipalName
                $rader.Beskrivelse = $gruppemedlem.Description
                $rader.ObjektId = $gruppemedlem.ObjectId
                $tabell += $rader
            }
            $tabell | Format-Table -AutoSize | Out-Host
        }
}
function listMembersMemberTypeDevice
{
    param([array]$valgtSikkerhetsgruppe, [array]$gruppemedlemmer)
        If($gruppemedlemmer -eq $null){
            Write-Host "Det er ingen AAD-maskiner i sikkerhetsgruppen", $valgtSikkerhetsgruppe[3], ":`n`n" -ForegroundColor Yellow
        }Else{
            Write-Host "Det er", $gruppemedlemmer.Count, "AAD-maskiner i sikkerhetsgruppen", $valgtSikkerhetsgruppe[3], ":`n" -ForegroundColor Green
            [int]$indeks = 0
            $tabell = @()
            ForEach($gruppemedlem in $gruppemedlemmer){
                $indeks++
                $rader = "" | Select Indeks,Navn,Operativsystem,OSVersjon,ObjektId,MaskinId
                $rader.Indeks = $indeks
                $rader.Navn = $gruppemedlem.DisplayName
                $rader.Operativsystem = $gruppemedlem.DeviceOSType
                $rader.OSVersjon = $gruppemedlem.DeviceOSVersion
                $rader.ObjektId = $gruppemedlem.ObjectId
                $rader.MaskinId = $gruppemedlem.DeviceId
                $tabell += $rader
            }
            $tabell | Format-Table -AutoSize | Out-Host
        }
}
function listAllAADGroups{
    loggedInUser
    Write-Host "`t`t`t - - Lister ut alle AAD-grupper - - `n`n" -ForegroundColor Magenta
    $alleSikkerhetsgrupper = Get-AzureADGroup | Select-Object DisplayName,MailNickName,Description,ObjectId 
    Write-Host "Det finnes følgende", $alleSikkerhetsgrupper.Count, "sikkerhetsgrupper i AAD:" -ForegroundColor Green
    $alleSikkerhetsgrupper | Format-Table -AutoSize | Out-Host
}
#-------------------------------- Gruppeadministrasjon - Slutt -------------------------------------
#-------------------------------- Maskinadministrasjon - Start -------------------------------------
function deviceAdministration{
    [array]$sjekkeOmFinnesAADMaskiner = Get-AzureADDevice | Select-Object * | Sort-Object DisplayName
    If($sjekkeOmFinnesAADMaskiner -eq $null){
        [string]$feilmelding = "Det er ingen maskiner i AAD!`n",`
                                "`t   Denne funksjonaliteten krever at det finnes maskiner i AAD.`n",`
                                "`t   Legg til noen AAD-maskiner, og prøv igjen"
        error $feilmelding
    }Else{
        [string]$Script:meny = "Maskinadministrasjon"
        [string]$tittel = "Maskinadministrasjon"
        [array]$alleMenyvalg = "Endre AAD-maskin",`
                               "Slett AAD-maskin",`
                               "List ut alle AAD-maskiner",`
                               "Vis utdypende informasjon om en AAD-maskin",`
                               "Tilbake til hovedmeny"
        Do{
            $maskinAdminMeny = listMenu $tittel $alleMenyvalg
        }While($maskinAdminMeny -lt 1 -or $maskinAdminMeny -gt $alleMenyvalg.Count)
        If($maskinAdminMeny -eq 1){changeAADDevice}
        ElseIf($maskinAdminMeny -eq 2){deleteAADDevice}
        ElseIf($maskinAdminMeny -eq 3){listAllAADDevices}
        ElseIf($maskinAdminMeny -eq 4){informationAADDevice}
        ElseIf($maskinAdminMeny -eq 5){mainMenu}
    }
    returnToMenu
}
function changeAADDevice{
    [string]$tittel = "Endre AAD-maskin"
    [array]$alleMenyvalg = "Endre navn på AAD-maskin",`
                           "Endre registrert eier av AAD-maskin",`
                           "Endre registrert bruker av AAD-maskin",`
                           "Avbryt" 
    Do{
        $brukervalg = listMenu $tittel $alleMenyvalg
    }While($brukervalg -lt 1 -or $brukervalg -gt $alleMenyvalg.Count)
    If($brukervalg -eq 1){changeNameAADDevice}
    ElseIf($brukervalg -eq 2){changeRegisteredOwnerOfDevice}
    ElseIf($brukervalg -eq 3){changeRegisteredUserOfDevice}
    ElseIf($brukervalg -eq 4){deviceAdministration}
    returnToMenu
}
function changeRegisteredOwnerOfDevice{
    [string]$tittel = "Endre registrert eier av AAD-maskin"
    [array]$alleMenyvalg = "Legg til eier av AAD-maskin",`
                           "Bytt ut en eier av AAD-maskin",`
                           "Fjern eier fra AAD-maskin",`
                           "Avbryt"
    Do{
        $brukervalg = listMenu $tittel $alleMenyvalg
    }While($brukervalg -lt 1 -or $brukervalg -gt $alleMenyvalg.Count)
    If($brukervalg -eq 1){addAADDeviceRegisteredOwner}
    ElseIf($brukervalg -eq 2){replaceRegisteredOwnerOfDevice}
    ElseIf($brukervalg -eq 3){removeAADDeviceRegisteredOwner}
    ElseIf($brukervalg -eq 4){deviceAdministration}
    returnToMenu
}
function changeRegisteredUserOfDevice{
    [string]$tittel = "Endre registrert bruker av AAD-maskin"
    [array]$alleMenyvalg = "Legg til registrert bruker av AAD-maskin",`
                           "Bytt ut en registrert bruker av AAD-maskin",`
                           "Fjern en registrert bruker av AAD-maskin",`
                           "Avbryt"
    Do{
        $brukervalg = listMenu $tittel $alleMenyvalg
    }While($brukervalg -lt 1 -or $brukervalg -gt $alleMenyvalg.Count)
    If($brukervalg -eq 1){addAADDeviceRegisteredUser}
    ElseIf($brukervalg -eq 2){replaceRegisteredUserOfDevice}
    ElseIf($brukervalg -eq 3){removeAADDeviceRegisteredUser}
    ElseIf($brukervalg -eq 4){deviceAdministration}
    returnToMenu
}
function changeNameAADDevice{
    [string]$tittel = "Endre navn på AAD-maskin"
    [array]$aktuelleAADMaskiner = listAADDevices $tittel #Returnerer et objekt med følgende kolonner:ObjectId,DisplayName,DeviceId
    If($aktuelleAADMaskiner -eq $null){
        operationAborted '"Endre navn på AAD-maskin"'   
    }Else{
        Do{
            [string]$tittel = "Velg den AAD-maskinen du ønsker å endre navn på"
            [string]$objekttype = "Maskin"
            [array]$valgtAADMaskin = chooseARow $tittel $objekttype $aktuelleAADMaskiner #0=ObjectId,1=DisplayName,2=DeviceId
            If($valgtAADMaskin -eq $null){break}
            Else{
                loggedInUser
                Write-Host "Du har valgt å endre navn på maskinen",$valgtAADMaskin[1], "`n" `
                -ForegroundColor Magenta
                [string]$infoTilBruker = "Nytt navn:"
                [string]$nyttNavnAADMaskin = fillThisVariableWithAProperValue $infoTilBruker
                If($nyttNavnAADMaskin -eq ''){break}
                Else{
                    [string]$operasjon = "Bekreft følgende operasjon"
                    $variabelbeskrivelse = @()
                    $variabelbeskrivelse += "MASKINEN", "BYTTER NAVN TIL:"
                    $variabelnavn = @()
                    $variabelnavn += $valgtAADMaskin[1], $nyttNavnAADMaskin
                    $menyvalget = confirmSelectedData $operasjon $variabelbeskrivelse $variabelnavn
                }
            }
        }While($menyvalget -eq 2)
        If($menyvalget -eq 1){
            [string]$forrigeNavnAADMaskin = $valgtAADMaskin[1]
            Set-AzureADDevice -ObjectId $valgtAADMaskin[0] -DisplayName $nyttNavnAADMaskin | Out-Null
            [string]$navnCSVFil = "C:\NyttNavnAAD-maskin.csv"
            [string]$dato,[string]$tidspunkt = ((Get-Date -DisplayHint DateTime).ToString()).Split(' ')
            [string]$rad1CSVFil = "ForrigeNavn;NyttNavn;Dato;Tidspunkt;"
            [string]$rad2CSVFil = "$forrigeNavnAADMaskin;$nyttNavnAADMaskin;$dato;$tidspunkt;"
            loggingChanges $navnCSVFil $rad1CSVFil $rad2CSVFil
            [string]$beskjed = "Maskinen", $forrigeNavnAADMaskin, "har nå byttet navn til", $nyttNavnAADMaskin
            operationSucceeded $beskjed
        }Else{operationAborted '"Endre navn på AAD-maskin"'}
    }
}
function listAllRegisteredOwnersOfDevice
{
    param([string]$objektId)
        [array]$alleRegistrerteEiere = Get-AzureADDeviceRegisteredOwner -ObjectId $objektId | Select-Object *
        return $alleRegistrerteEiere
}
function addAADDeviceRegisteredOwner{
    [int]$brukervalg = 0
    [array]$valgtAADMaskin = $null
    [array]$valgtAADBruker = $null
    [string]$tittel = "Endre registrert eier på AAD-maskin"
    [array]$aktuelleAADMaskiner = listAADDevices $tittel #Returnerer et objekt med følgende kolonner:ObjectId,DisplayName,DeviceId
    If($aktuelleAADMaskiner -ne $null){
        [string]$tittel = "Velg den AAD-maskinen du ønsker å legge til en registrert eier på"
        [string]$objekttype = "Maskin"
        $valgtAADMaskin = chooseARow $tittel $objekttype $aktuelleAADMaskiner #0=ObjectId,1=DisplayName,2=DeviceId
        If($valgtAADMaskin -eq $null){operationAborted '"Legg til en registrert eier av AAD-maskin"'}
        Else{
            [array]$eiere = listAllRegisteredOwnersOfDevice $valgtAADMaskin[0]
            [array]$alleAADBrukere = allAADUsers
            If($eiere -ne $null){
                ForEach($eier in $eiere.MailNickName){
                    $alleAADBrukere = $alleAADBrukere | Where-Object {$_.MailNickName -ne $eier}
                }
            }
            $tittel = "Velg den brukeren som skal legges til som eier av maskinen", $valgtAADMaskin[1]
            [string]$objekttype = "Bruker"
            Do{
                $valgtAADBruker = chooseARow $tittel $objekttype $alleAADBrukere #0=ObjectId,1=DisplayName,2=UserPrincipalName,3=GivenName,4=SurName
                If($valgtAADBruker -eq $null){break}
                Else{
                    [string]$operasjon = "Følgende bruker legges til som eier av", $valgtAADMaskin[1]
                    $variabelbeskrivelse = @()
                    $variabelbeskrivelse += "NAVN", "BRUKERNAVN"
                    $variabelnavn = @()
                    $variabelnavn += $valgtAADBruker[1], $valgtAADBruker[2]
                    $brukervalg = confirmSelectedData $operasjon $variabelbeskrivelse $variabelnavn
                }
            }While($brukervalg -eq 2)
            If($brukervalg -ne 1){operationAborted '"Legg til en registrert eier av AAD-maskin"'}
            Else{
                Add-AzureADDeviceRegisteredOwner -ObjectId $valgtAADMaskin[0] -RefObjectId $valgtAADBruker[0]
                [string]$beskjed = "Brukeren", $valgtAADBruker[2], "er nå lagt til som eier av", $valgtAADMaskin[1]
                operationSucceeded $beskjed        
            }
        }
    }Else{operationAborted '"Legg til en registrert eier av AAD-maskin"'}         
}
function replaceRegisteredOwnerOfDevice{
    [int]$bekreftelseFraBruker = 0
    [string]$beskjed = ""
    [bool]$feilmelding = $false
    [bool]$avbruttOperasjon = $false
    [array]$valgtEnhet = $null
    [array]$valgtAADBruker = $null
    [string]$tittel = "Velg den AAD-maskinen du skal erstatte en registrert eier på"
    [array]$valgtEnhet = listAADDevices $tittel #Returnerer et objekt med følgende kolonner:ObjectId,DisplayName,DeviceId
    If($valgtEnhet -ne $null){
        [string]$tittel = "Velg den maskinen du ønsker å bytte ut en eier på"
        [string]$objekttype = "Maskin"
        [array]$valgtEnhet = chooseARow $tittel $objekttype $valgtEnhet #0=ObjectId,1=DisplayName,2=DeviceId
        If($valgtEnhet -ne $null){
            [array]$eiere = listAllRegisteredOwnersOfDevice $valgtEnhet[0]
            If($eiere -eq $null){
                $feilmelding = $true
                $beskjed = "Ingen AAD-brukere er oppført som eier på maskinen", $valgtEnhet[1]
            }Else{
                Do{
                    [string]$tittel = "Velg brukeren du ønsker å erstatte av en annen bruker som eier av maskinen", $valgtEnhet[1]
                    [string]$objekttype = "Bruker"
                    [array]$valgtAADBruker = chooseARow $tittel $objekttype $eiere #0=ObjectId,1=DisplayName,2=UserPrincipalName,3=GivenName,4=SurName
                    If($valgtAADBruker -eq $null){
                        $avbruttOperasjon = $true
                        break
                    }Else{
                        [string]$operasjon = "Du har valgt at følgende bruker skal erstattes av en annen bruker som eier fra maskinen", $valgtEnhet[1]
                        $variabelbeskrivelse = @()
                        $variabelbeskrivelse += "NAVN", "BRUKERNAVN"
                        $variabelnavn = @()
                        $variabelnavn += $valgtAADBruker[1], $valgtAADBruker[2]
                        $bekreftelseFraBruker = confirmSelectedData $operasjon $variabelbeskrivelse $variabelnavn
                        If($bekreftelseFraBruker -eq 3){
                            $avbruttOperasjon = $true
                            break
                        }
                    }
                }While($bekreftelseFraBruker -eq 2)
                If($avbruttOperasjon -ne $true){
                    [array]$muligeNyeEiere = allAADUsers
                    ForEach($eier in $eiere){
                        $muligeNyeEiere = $muligeNyeEiere | Where-Object {$_.UserPrincipalName -ne $eier.UserPrincipalName}
                    }
                    If($muligeNyeEiere -eq $null){
                        $feilmelding = $true
                        [string]$beskjed = "Alle brukere er allerede oppført som eier av maskinen", $valgtEnhet[1]
                    }Else{

                        Do{
                            [string]$tittel = "Velg brukeren som skal erstatte brukeren", $valgtAADBruker[2], "som eier av maskinen", $valgtEnhet[1]
                            [array]$nyEier = chooseARow $tittel $objekttype $muligeNyeEiere #0=ObjectId,1=DisplayName,2=UserPrincipalName,3=GivenName,4=SurName
                            If($nyEier -eq $null){
                                $avbruttOperasjon = $true
                                break
                            }Else{
                                [string]$operasjon = "Du har valgt at følgende bruker skal erstatte brukeren",$valgtAADBruker[2], "som eier av enheten", $valgtEnhet[1]
                                $variabelbeskrivelse = @()
                                $variabelbeskrivelse += "NAVN", "BRUKERNAVN"
                                $variabelnavn = @()
                                $variabelnavn += $nyEier[1], $nyEier[2]
                                $bekreftelseFraBruker = confirmSelectedData $operasjon $variabelbeskrivelse $variabelnavn
                                If($bekreftelseFraBruker -eq 3){
                                    $avbruttOperasjon = $true
                                    break
                                }
                            }
                        }While($bekreftelseFraBruker -eq 2)
                        If($avbruttOperasjon -ne $true){
                            Remove-AzureADDeviceRegisteredOwner -ObjectId $valgtEnhet[0] -OwnerId $valgtAADBruker[0]
                            Add-AzureADDeviceRegisteredOwner -ObjectId $valgtEnhet[0] -RefObjectId $nyEier[0]
                            [string]$beskjed = "Brukeren", $nyEier[2], "har nå erstattet brukeren",$valgtAADBruker[2], "som eier av enheten", $valgtEnhet[1]
                            operationSucceeded $beskjed
                        }
                    }
                }
            }
        }Else{$avbruttOperasjon = $true}
    }Else{$avbruttOperasjon = $true}
    If($avbruttOperasjon -eq $true){operationAborted '"Erstatt registrert eier av AAD-maskin"'}
    If($feilmelding -eq $true){
        error $beskjed
    }
}
function removeAADDeviceRegisteredOwner{
    [bool]$avbruttOperasjon = $false
    [bool]$feilmelding = $false
    [string]$feilmeldingsinfo = ""
    [int]$bekreftelseFraBruker = 0
    [array]$valgtAADBruker = $null
    [string]$tittel = "Velg den AAD-maskinen du ønsker å fjerne en eier fra"
    [array]$valgtEnhet = listAADDevices $tittel #Returnerer et objekt med følgende kolonner:ObjectId,DisplayName,DeviceId
    If($valgtEnhet -ne $null){
        [string]$tittel = "Velg den maskinen du ønsker å fjerne en registrert eier fra"
        [string]$objekttype = "Maskin"
        [array]$valgtEnhet = chooseARow $tittel $objekttype $valgtEnhet #0=ObjectId,1=DisplayName,2=DeviceId
        If($valgtEnhet -eq $null){$avbruttOperasjon = $true}
        Else{
            [array]$eiere = listAllRegisteredOwnersOfDevice $valgtEnhet[0]
            If($eiere -eq $null){
                $feilmeldingsinfo = "Ingen AAD-brukere er oppført som eier av maskinen", $valgtEnhet[1]
            }Else{
                Do{
                    [string]$tittel = "Følgende AAD-brukere er registrert som eier av maskinen", $valgtEnhet[1],":`n`n",`
                                      "Velg brukeren som skal fjernes som registrert eier av maskinen"
                    [string]$objekttype = "Bruker"
                    [array]$valgtAADBruker = chooseARow $tittel $objekttype $eiere #0=ObjectId,1=DisplayName,2=UserPrincipalName,3=GivenName,4=SurName
                    If($valgtAADBruker -eq $null){
                        $avbruttOperasjon = $true
                        break
                    }
                    Else{
                        [string]$operasjon = "Følgende bruker skal fjernes som eier av maskinen", $valgtEnhet[1]
                        $variabelbeskrivelse = @()
                        $variabelbeskrivelse += "NAVN", "BRUKERNAVN"
                        $variabelnavn = @()
                        $variabelnavn += $valgtAADBruker[1], $valgtAADBruker[2]
                        $bekreftelseFraBruker = confirmSelectedData $operasjon $variabelbeskrivelse $variabelnavn
                        If($bekreftelseFraBruker -eq 3){
                            $avbruttOperasjon = $true
                            break
                        }
                    }
                }While($bekreftelseFraBruker -eq 2)
                If($avbruttOperasjon -eq $false){
                    Remove-AzureADDeviceRegisteredOwner -ObjectId $valgtEnhet[0] -OwnerId $valgtAADBruker[0]
                    [string]$beskjed = "Brukeren", $valgtAADBruker[2], "er nå fjernet som eier av", $valgtEnhet[1]
                    operationSucceeded $beskjed
                }
            }
        }
    }Else{$avbruttOperasjon = $true}
    If($avbruttOperasjon -eq $true){operationAborted '"Fjerne eier fra AAD-maskin"'}
    If($feilmelding -eq $true){
        error $feilmeldingsinfo
    }
}
function addAADDeviceRegisteredUser{
    [int]$brukervalg = 0
    [bool]$avbruttOperasjon = $false
    [array]$valgtAADBruker = $null
    [array]$valgtAADMaskin = $null
    [string]$tittel = "Velg den maskinen du ønsker å legge til en ny registrert bruker av"
    [array]$valgtEnhet = listAADDevices $tittel #Returnerer et objekt med følgende kolonner:ObjectId,DisplayName,DeviceId
    If($valgtEnhet -ne $null){
        Do{
            [string]$tittel = "Velg den maskinen du ønsker å legge til en ny registrert bruker av"
            [string]$objekttype = "Maskin"
            [array]$valgtAADMaskin = chooseARow $tittel $objekttype $valgtEnhet #0=ObjectId,1=DisplayName,2=DeviceId
            If($valgtAADMaskin -eq $null){
                $avbruttOperasjon = $true
                break
            }
            Else{
                [array]$brukereAvMaskin = listAllRegisteredUsersOfDevice $valgtAADMaskin[0]
                [array]$alleAADBrukere = allAADUsers
                ForEach($brukerAvMaskin in $brukereAvMaskin.MailNickName){
                    $alleAADBrukere = $alleAADBrukere | Where-Object {$_.MailNickName -ne $brukerAvMaskin}
                }
                [string]$tittel = "Legg til en registrert bruker av maskinen", $valgtAADMaskin[1]
                [string]$objekttype = "Bruker"
                [array]$valgtAADBruker = chooseARow $tittel $objekttype $alleAADBrukere #0=ObjectId,1=DisplayName,2=UserPrincipalName,3=GivenName,4=SurName
                If($valgtAADBruker -ne $null){
                    [string]$operasjon = "Følgende bruker legges til som bruker av", $valgtAADMaskin[1]
                    $variabelbeskrivelse = @()
                    $variabelbeskrivelse += "NAVN", "BRUKERNAVN"
                    $variabelnavn = @()
                    $variabelnavn += $valgtAADBruker[1], $valgtAADBruker[2]
                    $brukervalg = confirmSelectedData $operasjon $variabelbeskrivelse $variabelnavn
                    If($brukervalg -eq 3){
                        $avbruttOperasjon = $true
                        break
                    }
                }Else{
                    $avbruttOperasjon = $true
                    break
                }
            }
        }While($brukervalg -eq 2)
    }Else{$avbruttOperasjon = $true}
    If($avbruttOperasjon -eq $true){
        [string]$beskjed = '"Legg til registrert bruker på AAD-maskin"'
        operationAborted $beskjed
    }Else{
        Add-AzureADDeviceRegisteredUser -ObjectId $valgtAADMaskin[0] -RefObjectId $valgtAADBruker[0]
        [string]$beskjed = "Brukeren", $valgtAADBruker[2], "er nå registrert som bruker av maskinen", $valgtAADMaskin[1]
        operationSucceeded $beskjed
    }
}
function replaceRegisteredUserOfDevice{
    [bool]$avbruttOperasjon = $false
    [string]$tittel = "Velg den AAD-maskinen du ønsker å erstatte en registrert bruker med en ny registrert bruker på"
    [int]$bekreftelseFraBruker = 0
    [array]$valgtAADBruker = $null
    [array]$valgtAADMaskin = $null
    [array]$nyRegistrertBruker = $null
    [array]$erstattMeg = $null
    [array]$muligeAADMaskiner = listAADDevices $tittel #Returnerer et objekt med følgende kolonner:ObjectId,DisplayName,DeviceId
    If($muligeAADMaskiner -ne $null){
        Do{
            [string]$objekttype = "Maskin"
            [array]$valgtAADMaskin = chooseARow $tittel $objekttype $muligeAADMaskiner #0=ObjectId,1=DisplayName,2=DeviceId
            If($valgtAADMaskin -ne $null){
                [array]$registrerteBrukere = listAllRegisteredUsersOfDevice $valgtAADMaskin[0]
                $tittel = "Følgende AAD-brukere er registrert som brukere av maskinen", $valgtAADMaskin[1], "`n`n",`
                                  "Velg den registrerte bruker du ønsker å erstatte med en annen bruker"
                [string]$objekttype = "Bruker"
                $erstattMeg = chooseARow $tittel $objekttype $registrerteBrukere #0=ObjectId,1=DisplayName,2=UserPrincipalName,3=GivenName,4=SurName
                If($erstattMeg -ne $null){
                    [string]$operasjon = "Følgende bruker fjernes som registrert bruker og erstattes av en annen registrert bruker, på maskinen", $valgtAADMaskin[1]
                    $variabelbeskrivelse = @()
                    $variabelbeskrivelse += "NAVN", "BRUKERNAVN"
                    $variabelnavn = @()
                    $variabelnavn += $erstattMeg[1],$erstattMeg[2]
                    $bekreftelseFraBruker = confirmSelectedData $operasjon $variabelbeskrivelse $variabelnavn
                }
            }
        }While($bekreftelseFraBruker -eq 2)
        If($bekreftelseFraBruker -eq 1){
            Do{
                $tittel = "Velg den AAD-brukeren som skal erstatte", $erstattMeg[2], "som registrert bruker av maskinen", $valgtAADMaskin[1]
                [array]$alternativeErstattere = allAADUsers
                ForEach($registrertBruker in $registrerteBrukere){
                    $alternativeErstattere = $alternativeErstattere | Where-Object {$_.UserPrincipalName -ne $registrertBruker.UserPrincipalName}
                }
                [array]$nyRegistrertBruker = chooseARow $tittel $objekttype $alternativeErstattere #0=ObjectId,1=DisplayName,2=UserPrincipalName,3=GivenName,4=SurName
                If($nyRegistrertBruker -ne $null){
                    [string]$operasjon = "Følgende bruker erstatter",$erstattMeg[2], "som registrert bruker på maskinen",$valgtAADMaskin[1]
                    $variabelbeskrivelse = @()
                    $variabelbeskrivelse += "NAVN", "BRUKERNAVN"
                    $variabelnavn = @()
                    $variabelnavn += $nyRegistrertBruker[1],$nyRegistrertBruker[2]
                    $bekreftelseFraBruker = confirmSelectedData $operasjon $variabelbeskrivelse $variabelnavn
                    If($bekreftelseFraBruker -eq 3){$avbruttOperasjon = $true}
                }Else{$avbruttOperasjon = $true}
            }While($bekreftelseFraBruker -eq 2)
        }Else{$avbruttOperasjon = $true}
    }Else{$avbruttOperasjon = $true}
    If($avbruttOperasjon -eq $true){operationAborted '"Erstatt registrert bruker av AAD-maskin"'}
    Else{
        Remove-AzureADDeviceRegisteredUser -ObjectId $valgtAADMaskin[0] -UserId $erstattMeg[0]
        Add-AzureADDeviceRegisteredUser -ObjectId $valgtAADMaskin[0] -RefObjectId $nyRegistrertBruker[0]
        [string]$beskjed = "Brukeren", $nyRegistrertBruker[2], "har nå erstattet den registrerte brukeren",$erstattMeg[2], "som registrert bruker av maskinen", $valgtAADMaskin[1]
        operationSucceeded $beskjed         
    }
}
function removeAADDeviceRegisteredUser{
    [bool]$avbruttOperasjon = $false
    [string]$tittel = "Velg den AAD-maskinen du ønsker å fjerne en registrert bruker fra"
    [int]$bekreftelseFraBruker = 0
    [array]$valgtEnhet = $null
    [array]$valgtAADBruker = $null
    [array]$muligeAADMaskiner = listAADDevices $tittel #Returnerer et objekt med følgende kolonner:ObjectId,DisplayName,DeviceId
    If($muligeAADMaskiner -ne $null){
        [string]$tittel = "Velg den maskinen du ønsker å fjerne en registrert bruker fra"
        [string]$objekttype = "Maskin"
        $valgtEnhet = chooseARow $tittel $objekttype $muligeAADMaskiner #0=ObjectId,1=DisplayName,2=DeviceId
        If($valgtEnhet -ne $null){
            [array]$registrerteBrukereAvMaskinen = listAllRegisteredUsersOfDevice $valgtEnhet[0]
            Do{
                [string]$tittel = "Følgende AAD-brukere er registrert som bruker av maskinen", $valgtEnhet[1],":`n`n",`
                                  "Velg brukeren som skal fjernes som registrert bruker av maskinen"
                [string]$objekttype = "Bruker"
                $valgtAADBruker = chooseARow $tittel $objekttype $registrerteBrukereAvMaskinen #0=ObjectId,1=DisplayName,2=UserPrincipalName,3=GivenName,4=SurName
                If($valgtAADBruker -eq $null){
                    $avbruttOperasjon = $true
                    break
                }Else{
                    [string]$operasjon = "Følgende bruker skal fjernes som registrert bruker av maskinen", $valgtEnhet[1]
                    $variabelbeskrivelse = @()
                    $variabelbeskrivelse += "NAVN", "BRUKERNAVN"
                    $variabelnavn = @()
                    $variabelnavn += $valgtAADBruker[1], $valgtAADBruker[2]
                    $bekreftelseFraBruker = confirmSelectedData $operasjon $variabelbeskrivelse $variabelnavn
                    If($bekreftelseFraBruker -eq 3){
                        $avbruttOperasjon = $true
                        break
                    }
                }
            }While($bekreftelseFraBruker -eq 2)
        }Else{$avbruttOperasjon = $true}
    }Else{$avbruttOperasjon = $true}
    If($avbruttOperasjon -eq $true){operationAborted '"Fjern en registrert bruker fra AAD-maskin"'}
    Else{
        Remove-AzureADDeviceRegisteredUser -ObjectId $valgtEnhet[0] -UserId $valgtAADBruker[0]
        [string]$beskjed = "Brukeren", $valgtAADBruker[2], "er nå fjernet som registrert bruker av maskinen", $valgtEnhet[1]
        operationSucceeded $beskjed
    }
}
function listAllRegisteredUsersOfDevice
{
    param([string]$objektId)
    [array]$alleRegistrerteBrukere = Get-AzureADDeviceRegisteredUser -ObjectId $objektId | Select-Object *
    return $alleRegistrerteBrukere
}
function deleteAADDevice{
    [bool]$avbruttOperasjon = $false
    [string]$tittel = "Velg maskinen du ønsker å fjerne fra AAD"
    [int]$bekreftelseFraBruker = 0
    [array]$valgtEnhet = $null
    [array]$muligeAADMaskiner = listAADDevices $tittel #Returnerer et objekt med følgende kolonner:ObjectId,DisplayName,DeviceId
    If($muligeAADMaskiner -ne $null){
        Do{
            [string]$objekttype = "Maskin"
            [array]$valgtEnhet = chooseARow $tittel $objekttype $muligeAADMaskiner #0=ObjectId,1=DisplayName,2=DeviceId
            If($valgtEnhet -ne $null){
                [string]$operasjon = "Følgende maskin fjernes fra AAD"
                $variabelbeskrivelse = @()
                $variabelbeskrivelse += "NAVN"
                $variabelnavn = @()
                $variabelnavn += $valgtEnhet[1]
                $bekreftelseFraBruker = confirmSelectedData $operasjon $variabelbeskrivelse $variabelnavn
                If($bekreftelseFraBruker -eq 3){
                    $avbruttOperasjon = $true
                    break
                }
            }Else{
                $avbruttOperasjon = $true
                break
            }
        }While($bekreftelseFraBruker -eq 2)
    }Else{$avbruttOperasjon = $true}
    If($avbruttOperasjon -eq $true){
        operationAborted '"Slett AAD-maskin"'
    }Else{
        [string]$maskinnavn = $valgtEnhet[1]
        Remove-AzureADDevice -ObjectId $valgtEnhet[0]
        [string]$dato,[string]$tidspunkt = ((Get-Date -DisplayHint DateTime).ToString()).Split(' ')
        [string]$navnCSVFil = "C:\SlettedeAAD-maskiner.csv"
        [string]$rad1CSVFil = "Maskinnavn;Dato;Tidspunkt;"
        [string]$rad2CSVFil = "$maskinnavn;$dato;$tidspunkt;"
        loggingChanges $navnCSVFil $rad1CSVFil $rad2CSVFil
        cls
        [string]$beskjed = "Enheten", $maskinnavn, "er nå fjernet fra AAD"
        operationSucceeded $beskjed
    }
}
function listAllAADDevices{
    loggedInUser
    [array]$aadMaskiner = Get-AzureADDevice | Select-Object DisplayName,ObjectId,DeviceId,DeviceOsVersion,DeviceTrustType | Sort-Object DisplayName
        [int]$antallAADMaskiner = 0
        ForEach($aadMaskin in $aadMaskiner){
            $antallAADMaskiner += 1
        }
        [string]$antallAADMaskiner = ($antallAADMaskiner).ToString()
        Write-Host "- -",$antallAADMaskiner, "følgende maskiner er oppført i AAD - -`n`n" -ForegroundColor Magenta
        ForEach($aadMaskin in $aadMaskiner){
            If($aadMaskin.DeviceTrustType -eq "AzureAd"){
                Write-Host $aadMaskin.DisplayName, " er AAD-joinet:" -Separator '' -ForegroundColor Green
            }Else{Write-Host $aadMaskin.DisplayName, " er AAD-registrert:" -Separator '' -ForegroundColor Green}
                  $aadMaskin | Out-Host
        }
} 
function informationAADDevice{
    [string]$tittel = "Velg den AAD-maskinen du ønsker å se utdypende informasjon om"
    [array]$aktuelleAADMaskiner = listAADDevices $tittel #Returnerer et objekt med følgende kolonner:ObjectId,DisplayName,DeviceId
    If($aktuelleAADMaskiner -eq $null){operationAborted '"Vis utdypende informasjon om en AAD-maskin"'}
    Else{
        [string]$objekttype = "Maskin"
        [array]$valgtAADMaskin = chooseARow $tittel $objekttype $aktuelleAADMaskiner #0=ObjectId,1=DisplayName,2=DeviceId
        If($valgtAADMaskin -eq $null){operationAborted '"Vis utdypende informasjon om en AAD-maskin"'}
        Else{
            loggedInUser
            [array]$aktuellAADMaskin = Get-AzureADDevice -ObjectId $valgtAADMaskin[0] | Select-Object *
            Write-Host " - - Utdypende informasjon om", $aktuellAADMaskin.DisplayName, "- - `n`n" `
            -ForegroundColor Magenta
            If($aktuellAADMaskin.DeviceTrustType -eq "AzureAd"){
                Write-Host "Maskinen", $aktuellAADMaskin.DisplayName, "er AAD-joinet:" -ForegroundColor Green 
            }Else{
                Write-Host "Maskinen", $aktuellAADMaskin.DisplayName, "er AAD-registrert:" -ForegroundColor Green   
            }
            $aktuellAADMaskin | Out-Host
            Write-Host "Bruker(e) registrert som eier av", $aktuellAADMaskin.DisplayName, ":" -ForegroundColor Green
            Get-AzureADDeviceRegisteredOwner -ObjectId $aktuellAADMaskin.ObjectId | Out-Host
            Write-Host "Bruker(e) registrert som brukere av", $aktuellAADMaskin.DisplayName, ":" -ForegroundColor Green
            Get-AzureADDeviceRegisteredUser -ObjectId $aktuellAADMaskin.ObjectId | Out-Host
        }
    }
}
function listAADDevices
{
    param([string]$tittel)
    [array]$alleMenyvalg = "Velg fra utlisting av alle AAD-maskiner",
                           "Velg fra utlisting basert på søk",`
                           "Avbryt"
    Do{
        $brukervalget = listMenu $tittel $alleMenyvalg
    }While($brukervalget -lt 1 -or $brukervalget -gt $alleMenyvalg.Count)
    If($brukervalget -eq 3){return $null}
    ElseIf($brukervalget -eq 1){
        [array]$aadMaskiner = Get-AzureADDevice | Select-Object ObjectId,DisplayName,DeviceId | Sort-Object DisplayName
        return $aadMaskiner
    }ElseIf($brukervalget -eq 2){
        loggedInUser
        Do{
            [bool]$eksisterer = $true
            [string]$infoTilBruker = "Skriv inn første delen eller hele navnet til maskinen du ønsker å", $tittel
            $delAvMaskinnavn = fillThisVariableWithAProperValue $infoTilBruker
            If($delAvMaskinnavn -eq ''){return $null}
            Else{
                [array]$aadMaskiner = Get-AzureADDevice -SearchString $delAvMaskinnavn | Select -Property ObjectId,DisplayName,DeviceId | Sort-Object DisplayName
                If($aadMaskiner -eq $null){
                    cls
                    [string]$feilmelding = 'Det finnes ingen AAD-maskiner som begynner på "',$delAvMaskinnavn,'"!',`
                                           "`n`t   Prøv igjen"
                    error $feilmelding
                    $eksisterer = $false
                }
            }
        }While($eksisterer -eq $false)
        If($aadMaskiner -ne $null){return $aadMaskiner}
    }
}
#-------------------------------- Maskinadministrasjon - Slutt -------------------------------------
#-------------------------------- Lisensadministrasjon - Start -------------------------------------
#Oppsett av kode i henhold til lisenser er inspirert fra: [6]
function licenseAdministration{
    [array]$sjekkeOmNoenLisenser = Get-AzureADSubscribedSku
    If($sjekkeOmNoenLisenser -eq $null){
        [string]$feilmelding = "Det er ingen lisenser i AAD!`n",`
                                "`t   Denne funksjonaliteten krever at det finnes lisenser i AAD.`n",`
                                "`t   Legg til noen AAD-lisenser, og prøv igjen"
        error $feilmelding
    }Else{
        [string]$Script:meny = "Lisensadministrasjon"
        [string]$tittel = "Lisensadministrasjon"
        [array]$alleMenyvalg = "Alle lisenser",`
                               "Brukere og lisenser",`
                               "Grupper og lisenser",`
                               "Utdypende informasjon om en lisens",`
                               "Tilbake til hovedmeny"
        Do{
            $lisensAdminMeny = listMenu $tittel $alleMenyvalg
        }While($lisensAdminMeny -lt 1 -or $lisensAdminMeny -gt $alleMenyvalg.Count)
        loggedInUser
        If($lisensAdminMeny -eq 1){listAllLicenses}
        ElseIf($lisensAdminMeny -eq 2){licenseAADUser}
        ElseIf($lisensAdminMeny -eq 3){licenseAADGroup}
        ElseIf($lisensAdminMeny -eq 4){informationLicense}
        ElseIf($lisensAdminMeny -eq 5){mainMenu}  
    }
    returnToMenu
}
function listAllLicenses{
    [array]$alleLisenser = Get-AzureADSubscribedSku
    [int]$teller = 0
    Write-Host " - - Det er", $alleLisenser.Count, "lisens(er) tilknyttet aktuell Azure-portal - - `n`n" `
    -ForegroundColor Magenta
    Get-MsolAccountSku | Select-Object AccountSkuId,AccountName,SkuPartNumber,SkuId,SubscriptionIds,ConsumedUnits,`
    ActiveUnits,AccountObjectId | Format-Table -AutoSize | Out-Host
    Write-Host "`n"
    ForEach($lisens in $alleLisenser){
        $teller++
        Write-Host $teller, ". lisensen, ", $lisens.SkuPartNumber, ", inneholder ", ($lisens.ServicePlans).Count, " følgende tjeneste(r):" -ForegroundColor Green -Separator ''
        $tjenester = $lisens.serviceplans
        $tjenester | Sort-Object ServicePlanName -Unique | Out-Host
    }
}
function listLicenses
{
    param([string]$handling)
        [string]$tittel = "Velg den AAD-lisensen som du ønsker å", $handling
        [string]$objekttype = "Lisens"
        [array]$muligeAADLisenser = Get-MsolAccountSku | Select-Object SkuPartNumber,SkuId,ConsumedUnits,ActiveUnits,AccountSkuId
        [array]$valgtAADLisens = $null
        [int]$bekreftelseFraBruker = 0
        Do{
            [array]$valgtAADLisens = chooseARow $tittel $objekttype $muligeAADLisenser #0=SkuId,1=SkuPartNumber2=AccountSkuId
            If($valgtAADLisens -eq $null){
                return $null
            }Else{
                [string]$operasjon = "Følgende lisens benyttes som utgangspunkt for å", $handling
                $variabelbeskrivelse = @()
                $variabelbeskrivelse += "LISENSNAVN"
                $variabelnavn = @()
                $variabelnavn += $valgtAADLisens[1]
                $bekreftelseFraBruker = confirmSelectedData $operasjon $variabelbeskrivelse $variabelnavn
                If($bekreftelseFraBruker -eq 3){
                    $operasjonAvbrutt = $true
                }
                If($bekreftelseFraBruker -ne 2){break}
            }
        }While($bekreftelseFraBruker -eq 2)
        If($operasjonAvbrutt -eq $true){return $null}
        Else{return $valgtAADLisens}
}
function licenseAADUser{
    [string]$tittel = "AAD-lisenser og -brukere"
    [array]$alleMenyvalg = "Brukere som har lisens",`
                           "Brukere som mangler lisens",`
                           "Tildel lisens til bruker",`
                           "Avbryt"
    Do{
        $brukervalg = listMenu $tittel $alleMenyvalg
    }While($brukervalg -lt 1 -or $brukervalg -gt $alleMenyvalg.Count)
    [string]$handling = ""
    If($brukervalg -eq 1){
        $handling = "sjekke hvilke AAD-brukere er tildelt"
    }ElseIf($brukervalg -eq 2){
        $handling = "sjekke hvilke AAD-brukere som mangler"
    }ElseIf($brukervalg -eq 3){
        $handling = "tildele en AAD-bruker"
    }
    If($brukervalg -eq 4){licenseAdministration}
    Else{
        [array]$valgtAADLisens = listLicenses $handling #0=SkuId,1=SkuPartNumber,2=AccountSkuId
        [bool]$avbruttOperasjon = $false
        If($valgtAADLisens -eq $null){
            $avbruttOperasjon = $true
        }Else{
            [string]$operasjonVellykket = ""
            [string]$feilmelding = ""
            [bool]$noeGikkGalt = $false
            If($brukervalg -eq 1){
                [array]$aadBrukereSomHarLisensen = listLicensedAADUsers $valgtAADLisens
                If($aadBrukereSomHarLisensen -eq $null){
                    $feilmelding = "Ingen AAD-brukere er tildelt lisensen", $valgtAADLisens[1]
                    $noeGikkGalt = $true
                }Else{
                    loggedInUser
                    Write-Host " - - Følgende", $aadBrukereSomHarLisensen.Count, `
                    "AAD-brukere er tildelt lisensen", $valgtAADLisens[1], "- - `n" -ForegroundColor Magenta
                    $aadBrukereSomHarLisensen | Out-Host
                }
            }ElseIf($brukervalg -eq 2){
                [array]$aadBrukereSomManglerLisensen = listUnlicensedAADUsers $valgtAADLisens
                If($aadBrukereSomManglerLisensen -eq $null){
                    $feilmelding = "Alle AAD-brukere har allerede lisensen", $valgtAADLisens[1]
                    $noeGikkGalt = $true
                }Else{
                    loggedInUser
                    Write-Host " - - Følgende", $aadBrukereSomManglerLisensen.Count, `
                    "AAD-brukere mangler lisensen", $valgtAADLisens[1], "- - `n" `
                    -ForegroundColor Magenta
                    $aadBrukereSomManglerLisensen | Out-Host
                }
            }ElseIf($brukervalg -eq 3){
                [array]$aadBrukereSomHarLisensen = listLicensedAADUsers $valgtAADLisens
                If($aadBrukereSomHarLisensen -eq $null){
                    $noeGikkGalt = $true
                    $feilmelding = "Alle AAD-brukere har allerede lisensen", $valgtAADLisens[1]
                }Else{
                    $menyvalg = changeLicenseAADUser $valgtAADLisens "Tildel lisens til AAD-bruker" #parametre: [array]$lisens,[string]$beskrivelse #Fjern,erstatt eller legg til
                    If($menyvalg -eq 3){
                        $avbruttOperasjon = $true
                    }Else{
                        [string]$beskjed = "Velg AAD-brukeren som skal tildeles lisensen", $valgtAADLisens[1]
                        [array]$muligeAADBrukereAzureAD = allAADUsers
                        ForEach($aadBruker in $aadBrukereSomHarLisensen){
                            $muligeAADBrukereAzureAD = $muligeAADBrukereAzureAD | Where-Object {$_.UserPrincipalName -ne $aadBruker.UserPrincipalName}
                        }
                        If($menyvalg -eq 1){
                            $beskjed += " direkte"
                        }ElseIf($menyvalg -eq 2){
                            $beskjed += " indirekte"
                        }
                        [string]$tittel = $beskjed
                        [string]$objekttype = "Bruker"
                        [int]$bekreftelseFraBruker = 0
                        Do{
                            [array]$valgtAADBruker = chooseARow $tittel $objekttype $muligeAADBrukereAzureAD #0=ObjectId,1=DisplayName,2=UserPrincipalName,3=GivenName,4=SurName
                            If($valgtAADBruker -eq $null){$avbruttOperasjon = $true}
                            Else{
                                [string]$operasjon = "Følgende AAD-bruker tildeles lisensen", $valgtAADLisens[1]
                                $variabelbeskrivelse = @()
                                $variabelbeskrivelse += "NAVN", "BRUKERNAVN"
                                $variabelnavn = @()
                                $variabelnavn += $valgtAADBruker[1],$valgtAADBruker[2]
                                $bekreftelseFraBruker = confirmSelectedData $operasjon $variabelbeskrivelse $variabelnavn 
                                If($bekreftelseFraBruker -eq 3){$avbruttOperasjon = $true}
                                If($bekreftelseFraBruker -ne 2){break}
                            }
                        }While($bekreftelseFraBruker -eq 2)
                        If($menyvalg -eq 1 -and $bekreftelseFraBruker -eq 1){
                            Set-MsolUser -UserPrincipalName $valgtAADBruker[2] -UsageLocation "NO" | Out-Null
                            Set-MsolUserLicense -UserPrincipalName $valgtAADBruker[2]  -AddLicenses $valgtAADLisens[2]
                            $operasjonVellykket = "AAD-brukeren", $valgtAADBruker[2], "er nå tildelt lisensen", $valgtAADLisens[1], "direkte"
                            operationsucceeded $operasjonVellykket
                        }ElseIf($menyvalg -eq 2 -and $bekreftelseFraBruker -eq 1){
                            [array]$grupperUtenLisensen = listUnlicensedAADGroups $valgtAADLisens
                            If($grupperutenLisensen -eq $null){
                                $noeGikkGalt = $true
                                $feilmelding = "Ingen AAD-sikkerhetsgrupper er tildelt lisensen", $valgtAADLisens[1]
                            }Else{
                                [array]$muligeAADSikkerhetsgrupperAzureAD = Get-AzureADGroup | Select-Object ObjectId,DisplayName,Description,MailNickName
                                ForEach($gruppeUtenLisensen in $grupperUtenLisensen){
                                    $muligeAADSikkerhetsgrupperAzureAD = $muligeAADSikkerhetsgrupperAzureAD | Where-Object {$_.ObjectId -ne $gruppeUtenLisensen.ObjectId}
                                }
                                [string]$operasjon = " - - Følgende AAD-sikkerhetsgrupper er tildelt lisensen", $valgtAADLisens[1], "- -", `
                                                     "`n`nAAD-brukeren", $valgtAADBruker[2], "blir tildelt lisensen", $valgtAADLisens[1], `
                                                     "`nindirekte ved å plasseres i en sikkerhetsgruppe som er tildelt lisensen.", `
                                                     "`n`nVelg den AAD-sikkerhetsgruppen du ønsker å plassere brukeren", $valgtAADBruker[2], "inn i:"
                                [string]$objekttype = "Sikkerhetsgruppe"
                                [int]$bekreftelseFraBruker = 0
                                [array]$valgtAADSikkerhetsgruppe = $null
                                Do{
                                    $valgtAADSikkerhetsgruppe = chooseARow $operasjon $objekttype $muligeAADSikkerhetsgrupperAzureAD #0=ObjectId,1=DisplayName,2=MailNickName,3=Description
                                    If($valgtAADSikkerhetsgruppe -eq $null){
                                        $avbruttOperasjon = $true
                                        break
                                    }Else{
                                        [string]$operasjon = "Følgende AAD-bruker tildeles lisensen", $valgtAADLisens[1], `
                                                             "`nved å plasseres i sikkerhetsgruppen", $valgtAADSikkerhetsgruppe[1]
                                        $variabelbeskrivelse = @()
                                        $variabelbeskrivelse += "NAVN", "BRUKERNAVN"
                                        $variabelnavn = @()
                                        $variabelnavn += $valgtAADBruker[1],$valgtAADBruker[2]
                                        $bekreftelseFraBruker = confirmSelectedData $operasjon $variabelbeskrivelse $variabelnavn 
                                        If($bekreftelseFraBruker -eq 3){$avbruttOperasjon = $true}
                                        If($bekreftelseFraBruker -ne 2){break}
                                    }
                                }While($bekreftelseFraBruker -eq 2)
                                If($bekreftelseFraBruker -eq 1){
                                    Add-AzureADGroupMember -ObjectId $valgtAADSikkerhetsgruppe[0] -RefObjectId $valgtAADBruker[0]
                                    [string]$beskjed = "Brukeren", $valgtAADBruker[2], "har nå blitt tildelt lisensen", $valgtAADLisens[1], "via arv"
                                    operationSucceeded $beskjed
                                }
                            }
                        }
                    }
                }
            }
        }
        If($noeGikkGalt -eq $true){error $feilmelding}
        ElseIf($avbruttOperasjon -eq $true){operationAborted '"AAD-lisenser og -brukere"'}
    }
    returnToMenu
}
function listLicensedAADUsers
{
    param([array]$lisens)
        [array]$alleAADBrukereSomHarLisensen = Get-MsolUser -All | Where-Object {($_.Licenses).AccountSkuId -eq $lisens[2]}
        If($alleAADBrukereSomHarLisensen -eq $null){return $null}
        Else{return $alleAADBrukereSomHarLisensen}
}
function listUnlicensedAADUsers
{
    param([array]$lisens)
        [array]$alleAADBrukere = Get-MsolUser -All
        [array]$alleAADBrukereSomHarLisensen = $alleAADBrukere | `
                                               Where-Object {($_.Licenses).AccountSkuId -eq $lisens[2]}
        [array]$alleAADBrukereSomManglerLisensen = $alleAADBrukere
        ForEach($aadBruker in $alleAADBrukereSomHarLisensen){
            $alleAADBrukereSomManglerLisensen = $alleAADBrukereSomManglerLisensen | `
                                                Where-Object {$_.UserPrincipalName -ne $aadBruker.UserPrincipalName}
        }
        If($alleAADBrukereSomManglerLisensen -eq $null){return $null}
        Else{return $alleAADBrukereSomManglerLisensen}

}
function changeLicenseAADUser
{
    param([array]$lisens,[string]$beskrivelse)
        [string]$tittel = $beskrivelse #Fjern,erstatt eller legg til
        [array]$alleMenyvalg = "Direkte",`
                               "Indirekte (arv)",`
                               "Avbryt"
        Do{
            $brukervalg = listMenu $tittel $alleMenyvalg
        }While($brukervalg -lt 1 -or $brukervalg -gt $alleMenyvalg.Count)
        return $brukervalg     
}
function licenseAADGroup{
    [string]$tittel = "AAD-lisenser og -sikkerhetsgrupper"
    [array]$alleMenyvalg = "Grupper som har lisens",`
                           "Grupper som mangler lisens",`
                           "Avbryt"
    Do{
        $brukervalg = listMenu $tittel $alleMenyvalg
    }While($brukervalg -lt 1 -or $brukervalg -gt $alleMenyvalg.Count)
    [string]$handling = ""
    If($brukervalg -eq 1){
        $handling = "sjekke hvilke AAD-sikkerhetsgrupper er tildelt"
    }ElseIf($brukervalg -eq 2){
        $handling = "sjekke hvilke AAD-sikkerhetsgrupper som mangler"
    }
    If($brukervalg -eq 3){licenseAdministration}
    Else{
        [array]$valgtAADLisens = listLicenses $handling #0=SkuId,1=SkuPartNumber,2=AccountSkuId
        [bool]$avbruttOperasjon = $false
        If($valgtAADLisens -eq $null){
            $avbruttOperasjon = $true
        }Else{
            cls
            [string]$operasjonVellykket = ""
            [string]$feilmelding = ""
            [bool]$noeGikkGalt = $false
            If($brukervalg -eq 1){
                [array]$aadSikkerhetsgrupperSomHarLisensen = listLicensedAADGroups $valgtAADLisens
                If($aadSikkerhetsgrupperSomHarLisensen -eq $null){
                    $feilmelding = "Ingen AAD-sikkerhetsgrupper er tildelt lisensen", $valgtAADLisens[1]
                    $noeGikkGalt = $true
                }Else{
                    loggedInUser
                    Write-Host " - - Følgende", $aadSikkerhetsgrupperSomHarLisensen.Count, `
                    "stk. AAD-sikkerhetsgruppe(r) er tildelt lisensen", $valgtAADLisens[1], "- - `n" -ForegroundColor Magenta
                    $aadSikkerhetsgrupperSomHarLisensen | Format-Table -AutoSize -Property DisplayName,Description | Out-Host
                }
            }ElseIf($brukervalg -eq 2){
                [array]$aadSikkerhetsgrupperSomManglerLisensen = listUnlicensedAADGroups $valgtAADLisens
                If($aadSikkerhetsgrupperSomManglerLisensen -eq $null){
                    $feilmelding = "Alle AAD-sikkerhetsgrupper har allerede lisensen", $valgtAADLisens[1]
                    $noeGikkGalt = $true
                }Else{
                    loggedInUser
                    Write-Host " - - Følgende", $aadSikkerhetsgrupperSomManglerLisensen.Count, `
                    "stk. AAD-sikkerhetsgruppe(r) mangler lisensen", $valgtAADLisens[1], "- - `n" `
                    -ForegroundColor Magenta
                    $aadSikkerhetsgrupperSomManglerLisensen | Format-Table -AutoSize -Property DisplayName,Description | Out-Host
                }
            }
        }
        If($noeGikkGalt -eq $true){error $feilmelding}
        ElseIf($avbruttOperasjon -eq $true){operationAborted '"AAD-lisenser og -sikkerhetsgrupper"'}
        }
    returnToMenu
}
function listLicensedAADGroups
{
    param([array]$valgtAADLisens)
        [array]$aadSikkerhetsgruppenSomHarLisensen = Get-MsolGroup -GroupType Security | Select-Object * `
                                                    | where-object {$_.licenses.skupartnumber -eq $valgtAADLisens[1]}
        If($aadSikkerhetsgruppenSomHarLisensen -eq $null){return $null}
        Else{return $aadSikkerhetsgruppenSomHarLisensen}
}
function listUnlicensedAADGroups
{
    param([array]$valgtAADLisens)
        [array]$sikkerhetsgrupperSomManglerLisensen = Get-MsolGroup -GroupType Security | Select-Object * `
                                                     | where-object {$_.licenses.skupartnumber -ne $valgtAADLisens[1]}
        If($sikkerhetsgrupperSomManglerLisensen -eq $null){return $null}
        Else{return $sikkerhetsgrupperSomManglerLisensen}
}
function informationLicense{
    [string]$handling = "se utdypende informasjon om"
    [array]$valgtAADLisens = listLicenses $handling
    If($valgtAADLisens -eq $null){
        operationAborted '"Utdypende informasjon om en lisens"'
    }Else{
        loggedInUser
        Write-Host " - - Utdypende informasjon om AAD-lisensen", $valgtAADLisens[1], "- - `n`n" -ForegroundColor Magenta
        Write-Host $valgtAADLisens[1], ":`n" -Separator '' -ForegroundColor Green
        [array]$heleObjektet = Get-MsolAccountSku | Select-Object AccountSkuId,AccountName,SkuPartNumber,SkuId,SubscriptionIds,ConsumedUnits,`
                               ActiveUnits,AccountObjectId | Where-Object {$_.SkuPartNumber -eq $valgtAADLisens[1]}
        $heleObjektet | Format-Table -AutoSize | Out-Host
        [array]$inkluderteTjenester = Get-AzureADSubscribedSku | Where-Object {$_.SkuId -eq $heleObjektet.SkuId}
        $tjenester = $inkluderteTjenester.ServicePlans
        $tjenester = $tjenester | Sort-Object ServicePlanName -Unique
        Write-Host $valgtAADLisens[1], "inneholder", $tjenester.Count, "følgende tjenester:`n" -ForegroundColor Green
        $tjenester | Format-Table -AutoSize -Property ServicePlanName,AppliesTo,ProvisioningStatus,ServicePlanId | Out-Host
        [array]$aadBrukereSomHarLisensen = listLicensedAADUsers $valgtAADLisens
        If($aadBrukereSomHarLisensen -eq $null){
            Write-Host "Ingen AAD-brukere er tildelt lisensen",$valgtAADLisens[1],"- -`n`n" -ForegroundColor Yellow
        }Else{
            Write-Host "Følgende ",$aadBrukereSomHarLisensen.Count, " AAD-bruker(e) har lisensen ", `
            $valgtAADLisens[1], ":`n" -ForegroundColor Green -Separator ''
            $aadBrukereSomHarLisensen | Out-Host
        }
        [array]$aadSikkerhetsgrupperSomHarLisensen = listLicensedAADGroups $valgtAADLisens
        If($aadSikkerhetsgrupperSomHarLisensen -eq $null){
            Write-Host "Ingen AAD-sikkerhetsgrupper er tildelt lisensen",$valgtAADLisens[1],"- -" -ForegroundColor Yellow
        }Else{
            Write-Host "Følgende ",$aadSikkerhetsgrupperSomHarLisensen.Count, " AAD-sikkerhetsgruppe(r) har lisensen ", `
            $valgtAADLisens[1], ":`n" -ForegroundColor Green -Separator ''
            $aadSikkerhetsgrupperSomHarLisensen | Format-Table -AutoSize `
            -Property DisplayName,GroupType,Description,ObjectId | Out-Host
        }
    }
}
#-------------------------------- Lisensadministrasjon - Slutt -------------------------------------
connectToAzure